## MCMC for Estimating Systemic Risk Allocations 
#' @author Takaaki Koike and Marius Hofert
#' @date 2019November 24th

#' @Packages
library(actuar) # for simulating Pareto distribution
library(copula) # for copula objects
library(fitdistrplus) # for fitting Pareto distribution
library(gPdtest) # for fitting GPD
library(QRM) # for GPD functions  
library(rgl) # for 3d plots
library(xtable) # for outputting tables


##################################################################
### Model Parameters
##################################################################

d <- 3 # dimension for simulation study
dd <- 2 # dimension for empiricaly study

## Marginal distributions

## Student t 
nu <- 5 # degree of freedom for t distribution
tau <- 1 # scale
pF.t <- rep(list(function(x) { pt(x / tau, df = nu) }), d) # cdfs of student's t
qF.t <- rep(list(function(q) { tau * qt(q, df = nu) }), d) # quantiles of student's t
dF.t <- rep(list(function(x) { dt(x, df = nu) / tau }), d) # pdfs of student's t

## Generalized Pareto (GPD)
xi <- 0.3    # shape
sigma <- 1    # scale
pF.GPD <- rep(list(function(x) { pGPD(x, xi = xi, beta = sigma)}), d) # cdfs of GPD
qF.GPD <- rep(list(function(q) { qGPD(q, xi = xi, beta = sigma)}), d) # quantiles of GPD
dF.GPD <- rep(list(function(x) { dGPD(x, xi = xi, beta = sigma)}), d) # pdfs of GPD

## Copulas

## Clayton
theta <- 2  # parameter of Clayton copula with Kendall's tau = theta/(theta + 2)
cop.class.Cl <- archmCopula("clayton", dim = d, param = theta) 
# psi(t)=(1+t)^{-1/theta} with lower tail dependence coefficient 2^{-1/theta}
# Cl(u) propto (u^(-theta)+v^(-theta) -1 )^(-1/theta)

# one can check the Kendall's tau 0.5
#cdata <- 1-rCopula(1000,cop.class.Cl)
#Kendall(cdata)

## t Copula
nu.C <- 5 # degree of freedom
P <- diag(1,d) # correlation matrix
Pvec <- c() # off-diagonal entries of P 
for(i in 1:(d-1)){
  for(j in (i+1):d){    
    Pvec <- c(Pvec,abs(i-j)/(d)) 
    P[i,j] <- abs(i-j)/d
    P[j,i] <- abs(i-j)/d
  }
}  
P.inv <- solve(P)  # inverse of P
cop.class.t <- ellipCopula("t", dim = d, param = Pvec, dispstr = "un", df=nu.C)

# Variance matrix of multivariate Student t
Sig.t <- diag(tau, d) %*% P %*% diag(tau, d)
Sig.inv.t <- solve(Sig.t) # inverse

## Marginal and copula for empirical study
data(loss)
lambda1 <- 14036
theta1 <- 1.122
lambda2 <- 14219
theta2 <- 2.118
alpha.Cl <- 0.512

pF.pareto <- list(
      function(x) { ppareto(x, shape = theta1, scale = lambda1)},
      function(x) { ppareto(x, shape = theta2, scale = lambda2)}
      ) # cdf
qF.pareto <- list(
      function(q) { qpareto(q, shape = theta1, scale = lambda1)},
      function(q) { qpareto(q, shape = theta2, scale = lambda2)}
      ) # quantiles of student's t
dF.pareto <- list(
      function(x) { dpareto(x, shape = theta1, scale = lambda1)},
      function(x) { dpareto(x, shape = theta2, scale = lambda2)}
      ) # pdfs of student's t

cop.class.Cl.emp <-  archmCopula("clayton", dim = dd, param = alpha.Cl)

## levels in condition sets
alpha <- 0.99
alpha1 <- 0.975
alpha2 <- 0.99

## Levels in risk statistics
beta <- 0.99
beta1 <- 0.975
beta2 <- 0.99


##################################################################
### Define supplementary functions
##################################################################

#' @title Componentwise apply; apply j-th function to j-th column
#' @param x vector or matrix whose column size is d
#' @param Flist d list of functions
#' @return ( Flist[[j]]( x[,j] ) ) for j = 1,...,d
capply <- function(x, Flist) { 

  d <- length(Flist)
  x.col <- rbind(1:d, x) # the first row of x.col represents the column numbers
  rslt <- apply(x.col, 2, function(x.col){ Flist[[x.col[1]]](x.col[-1])})

  return(rslt)

}

#' @title Componentwise VaR
#' @param x matrix
#' @param alpha
#' @return (VaR_alpha(x[j],j=1,...,d)
ApplyVaR <- function(x, alpha) {   
  apply(x,2,function(x){quantile(x,alpha)}) 
}


#' @title Pick up marginal samples falling in certain ranges
#' @param x matrix
#' @param levels two quantile levels
#'       one level => {S = VaR1} 
#'       two levels => {VaR1 < S < VaR2}
#'       two levels with the second equals 1 => {S > VaR1}
#' @return x[,j] falling in [VaR1,j,VaR2,j]
RangeSample <- function(x, levels){

  N <- dim(x)[1]
  rslt <- apply(x,2,function(x){  sort(x)[(N*levels[1]):(N*levels[2])]  })

  return(rslt)

}

#' @title Kernel density estimation 
#' @param x vector
#' @param q quantile
#' @return 
DensEst = function(x, q){

  sigma <- sd(x)
  n <- length(x)
  h <- 1.06 * sigma * n ^ {- 1 / 5} # silverman's rule of thumb

  rslt <- mean(dnorm((x-q) / h)) / h

  return(rslt)

}

#' @title Componentwise kernel density estimation 
#' @param x matrix
#' @param q quantiles
#' @return componentwise kernel density estimators
cDensEst = function(x, q){

  d <- dim(x)[2]
  rslt <- 1:d 
  for(j in 1:d){
    rslt[j] <- DensEst(x[,j],q[j])
  }

  return(rslt)

}



##################################################################
### Monte Carlo Simulation
##################################################################

##' @title Monte Carlo Simulation
#' @param N: MC sample size
#' @param Copula: copula class
#' @param qF: list of marignal quantiles 
#' @param levels: confidence levels in the condition set
#'       one level => {S = VaR1} 
#'       two levels => {VaR1 < S < VaR2}
#'       two levels with the second equals 1 => {S > VaR1}
#' @return 
#'    MCsample: Monte Carlo samples in the condition set
#'      VaRests: MC estimates of Value-at-Risks
#'      RunTime: run time
SimulateMC <- function(N, Copula, qF, levels) {

  delta <- 0.001 # modification of the level of VaR crisis event

  set.seed(1221)  # Set seed

  start.time <- Sys.time()

  # simulate copula
  udata <- 1-rCopula(N, Copula) 
  # Note: I rotate for Clayton. Rotation does not matter for t copula.

  xdata <- capply(udata, qF)
  sdata <- rowSums(xdata)

  # MC samples in {S = VaR1}
  if(length(levels) == 1){
    levels.L <- levels - delta
    levels.U <- levels + delta
    # Note: I take samples falling in the 0.1% window around VaR1
    valpha <- sort(sdata)[N * levels]
    valpha.L <- sort(sdata)[N * levels.L]
    valpha.U <- sort(sdata)[N * levels.U]
    MCsample <- xdata[( valpha.L < sdata ) & (sdata <= valpha.U),]

    # VaR estimate
    valpha1 <- valpha
    valpha2 <- valpha
  }

  # MC samples in {VaR1 < S < VaR2}
  if(length(levels)==2){
    valpha.L <- sort(sdata)[N * levels[1]]
    valpha.U <- sort(sdata)[N * levels[2]]
    MCsample <- xdata[( valpha.L < sdata ) & (sdata <= valpha.U),]

    # VaR estimates
    valpha1 <- valpha.L
    valpha2 <- valpha.U

  }

  end.time <- Sys.time()
  runtime <- end.time - start.time

  return(list(
    MCsample = MCsample,
    VaRests = c(valpha1,valpha2),
    runtime = runtime
    ))
} 


## (1) GPD + Clayton
VaR.MC.GPD.Cl <- SimulateMC(N = 10^5, Copula = cop.class.Cl, qF = qF.GPD, levels = alpha) # {S = VaR}
RVaR.MC.GPD.Cl <- SimulateMC(N = 10^5, Copula = cop.class.Cl, qF = qF.GPD, levels = c(alpha1,alpha2))  # {Va1 < S < VaR2}
ES.MC.GPD.Cl <- SimulateMC(N = 10^5, Copula = cop.class.Cl, qF = qF.GPD, levels = c(alpha2,1)) # {VaR < S}

## (2) multivariate t 
VaR.MC.t.t <- SimulateMC(N = 10^5, Copula = cop.class.t, qF = qF.t, levels = alpha) # {S = VaR}
RVaR.MC.t.t <- SimulateMC(N = 10^5, Copula = cop.class.t, qF = qF.t, levels = c(alpha1,alpha2)) # {Va1 < S < VaR2}
ES.MC.t.t <- SimulateMC(N = 10^5, Copula = cop.class.t, qF = qF.t, levels = c(alpha2,1)) #  {VaR < S}

## (3) Empirical study 
VaR.MC.pareto.Cl <- SimulateMC(N = 10^5, Copula = cop.class.Cl.emp, qF = qF.pareto, levels = alpha) # {S = VaR}
RVaR.MC.pareto.Cl <- SimulateMC(N = 10^5, Copula = cop.class.Cl.emp, qF = qF.pareto, levels = c(alpha1,alpha2))  # {Va1 < S < VaR2}
ES.MC.pareto.Cl <- SimulateMC(N = 10^5, Copula = cop.class.Cl.emp, qF = qF.pareto, levels = c(alpha2,1)) # {VaR < S}


# data and simulated data plot
set.seed(1221)
udata <- 1-rCopula(10^5, cop.class.Cl)
xdata <- capply(udata, qF.pareto)
sdata <- rowSums(xdata)
v1 <- sort(sdata)[alpha1 * 10^5]
v2 <-  sort(sdata)[alpha2 * 10^5]

max.loss <- apply(loss[,1:2],2,max)
plot(xdata[1:1500,], xlim = c(0,max.loss[1]), ylim = c(0,max.loss[2]), xlab = "Loss", ylab= "Expense")
points(loss,col="blue")
abline(a = v1,b = -1,col="red")
abline(a = v2,b = -1,col="red")
legend("topright", legend  =  c("Data","Simulated sample"),pch = c(1,1), cex = 1.2, pt.cex = 1.2, col = c("black","blue"), bty = "o")



##################################################################
### Monte Carlo Estimation
##################################################################


#' @title Risk Statistics (Expectation, VaR, RVaR, ES) for MC conditional sample
#' @param x: Conditioned sample
#' @param levels.VaR : Confidence level for VaR
#' @param levels.RVaR : Confidence levels for RVaR
#' @param levels.ES : Confidence levels for ES
#' @return 
#'     Estimates of Mean, RVaR, VaR, ES 
#'     Standard errors of Mean, RVaR, VaR, ES 
RiskStats.MC <- function(
  x,
  level.VaR = beta,
  levels.RVaR = c(beta1, beta2),
  levels.ES = c(beta2, 1)
){

  # Expectation 
  if(class(x)=="numeric"){ # if there is only one sample
    d <- length(x)
    EXest <- x
    EXse <- rep(0,d)
  }else{  
    N <- dim(x)[1]
    d <- dim(x)[2]
    EXest <- colMeans(x)
    EXse <- apply(x,2,sd) / sqrt(N)
  }

  # VaR
  VaRest <- ApplyVaR(x,level.VaR)
  VaRse <- sqrt(level.VaR * (1 - level.VaR) / N) / cDensEst(x,VaRest)

  # RVaR
  sample.RVaR <- RangeSample(x, levels.RVaR)
  if(class(sample.RVaR) == "numeric") {
    RVaRest <- sample.RVaR
    RVaRse <- rep(0,d)
  } else {
    RVaRest <- colMeans(sample.RVaR)
    RVaRse <- apply(sample.RVaR,2,sd) / sqrt(dim(sample.RVaR)[1])
  }
  
  # ES
  sample.ES <- RangeSample(x, levels.ES)
  if(class(sample.ES)=="numeric") {
    ESest <- sample.ES
    ESse <- rep(0,d)
  } else {
    ESest <- colMeans(sample.ES)
    ESse <- apply(sample.ES,2,sd) / sqrt(dim(sample.ES)[1])
  }

  return(list(
    Meanest = EXest,
    Meanse = EXse,
    RVaRest = RVaRest,
    RVaRse = RVaRse,
    VaRest = VaRest,
    VaRse = VaRse,
    ESest = ESest,
    ESse = ESse
  ))
}


## (1) Pareto + Clayton
VaR.MC.stats.GPD.Cl <- RiskStats.MC(VaR.MC.GPD.Cl$MCsample) # {S = VaR}
RVaR.MC.stats.GPD.Cl <- RiskStats.MC(RVaR.MC.GPD.Cl$MCsample)  # {Va1 < S < VaR2}
ES.MC.stats.GPD.Cl <- RiskStats.MC(ES.MC.GPD.Cl$MCsample) # {VaR < S}

## (2) multivariate t
VaR.MC.stats.t.t <- RiskStats.MC(VaR.MC.t.t$MCsample) # {S = VaR}
RVaR.MC.stats.t.t <- RiskStats.MC(RVaR.MC.t.t$MCsample)  # {Va1 < S < VaR2}
ES.MC.stats.t.t <- RiskStats.MC(ES.MC.t.t$MCsample) #  {VaR < S}

## (3) Empirical study
VaR.MC.stats.pareto.Cl <- RiskStats.MC(VaR.MC.pareto.Cl$MCsample) # {S = VaR}
RVaR.MC.stats.pareto.Cl <- RiskStats.MC(RVaR.MC.pareto.Cl$MCsample)  # {Va1 < S < VaR2}
ES.MC.stats.pareto.Cl <- RiskStats.MC(ES.MC.pareto.Cl$MCsample) # {VaR < S}



##################################################################
### MCMC Estimation
##################################################################

##################################################################
### Target distributions
##################################################################


# Potential energy of GPD
UGPD <- function(x) {
  (1 + 1 / xi) * log(1 + xi * x / sigma)
}

# Gradient of UGPD
DUGPD <- function(x) {
  (1 + 1 / xi) * xi / (sigma + xi * x)
}

# Potential energy of Student t
Ut <- function(x) {
  (nu + 1) / 2 * log(1 + x^2 / nu / tau^2)
}

# Gradient of Ut
DUt <- function(x) {
  (nu + 1)* x / (nu * tau^2 + x^2)
}

# Potential energy of multivariate Student t
Umvt <- function(x) {
  d <- length(x)
  (nu + d) / 2 * log(1 + 1 / nu * t(x) %*% Sig.inv.t %*% x)
}

# Gradient of Umvt
DUmvt <- function(x) {
  d <- length(x)
  (nu + d) *  Sig.inv.t %*% x  / as.numeric(1 + 1 / nu * t(x) %*% Sig.inv.t %*% x)
}

# Potential energy of Student t copula
Utcop <- function(u){
  d <- length(u)
  x <- qt(u, df = nu.C)
  (nu.C + d) / 2 * log(1 + 1 / nu.C * t(x) %*% P.inv %*% x) - sum( (nu.C + 1) / 2 * log(1 + x^2 / nu.C / 1^2) )
}

# Gradient of Utcop
DUtcop <- function(u){
  d <- length(u)
  x <- qt(u, df = nu)
  sigma <- 1
  rslt <- ( (nu + d) *  P.inv %*% x  / as.numeric(1 + 1 / nu.C * t(x) %*% P.inv %*% x) - (nu + 1) * x / (nu.C * 1^2 + x^2) ) / dt(x, df = nu.C)
  return(as.vector(rslt))
}

# Potential energy of rotated Clayton copula
UrCl <- function(u) {
  d <- length(u)
  v <- 1-u
  (theta + 1) * sum(log(v)) + (d + 1 / theta) * log( - d + 1 + sum(v ^ (- theta)) )
}

# Gradient of UrCl
DUrCl <- function(u) {
  d <- length(u)
  v <- 1-u
  - (theta + 1) / v + (d + 1 / theta) * theta * v ^ (- theta - 1) / (- d + 1 + sum(v ^ (- theta)))
}

# (1) GPD + rotated Clayton
U.GPD.Cl <- function(x){
  u <- capply(x, pF.GPD)
  UrCl(u) + sum(UGPD(x))
}

DU.GPD.Cl <- function(x){
  u <- capply(x, pF.GPD)
  DUrCl(u) * capply(x, dF.GPD) + DUGPD(x)
}



# Potential energy of GPD
Upar1 <- function(x) {
  (theta1 + 1)* log(x + lambda1)
}

Upar2 <- function(x) {
  (theta2 + 1)* log(x + lambda2)
}


# Gradient of UGPD
DUpar1 <- function(x) {
  (theta1 + 1) /  (x + lambda1)
}

DUpar2 <- function(x) {
  (theta2 + 1) /  (x + lambda2)
}


# Potential energy of rotated Clayton copula
UrCl.emp <- function(u) {
  theta <- alpha.Cl
  d <- length(u)
  v <- 1-u
  (theta + 1) * sum(log(v)) + (d + 1 / theta) * log( - d + 1 + sum(v ^ (- theta)) )
}

# Gradient of UrCl
DUrCl.emp <- function(u) {
  theta <- alpha.Cl
  d <- length(u)
  v <- 1-u
  - (theta + 1) / v + (d + 1 / theta) * theta * v ^ (- theta - 1) / (- d + 1 + sum(v ^ (- theta)))
}

# (1) GPD + rotated Clayton
U.par.Cl <- function(x){
  u <- capply(x, pF.pareto)
  UrCl.emp(u) + Upar1(x[1]) + Upar2(x[2]) 
}

DU.par.Cl <- function(x){
  u <- capply(x, pF.pareto)
  DUrCl.emp(u) * capply(x, dF.pareto) + c(DUpar1(x[1]),DUpar2(x[2]))
}

# Kinetic energy distributions
K <- function(p){ 0.5 * t(p) %*% p } # Kinetic energy
DK <- function(p) { p } 

##################################################################
### Supplementary functions for MCMC
##################################################################


##' @title Batch means estimator of the standard deviation for the MCMC estimator
##' @param X N- sample path of the marginal Markov chain ( 1-dimension)
##' @return batch means estimator of the variance matrix
BMest <- function(X)
{
  N <- length(X)

  L <- floor(N^{1/2})   # batch length
  B <- floor(N / L)     # number of batches
  LB = L*B

  BatchMeans <- colMeans(matrix(X[1:LB], nrow = L))   # batch means
  GMean <- mean(X)  # global mean

  # variance
  rslt.BM <- {L * sum((BatchMeans - GMean)^2)} / (B-1)  # batch means estimator
  #standard deviation
  rslt.BM <- sqrt(rslt.BM)

  return(rslt.BM)

}


#' @title Risk Statistics (Expectation, VaR, RVaR, ES) for MCMC sample path
#' @param X: Sample path from MCMC
#' @param levels.VaR : Confidence level for VaR
#' @param levels.RVaR : Confidence levels for RVaR
#' @param levels.ES : Confidence levels for ES
#' @return 
#'     Estimates of Mean, RVaR, VaR, ES 
#'     Standard errors of Mean, RVaR, VaR, ES 
RiskStats.MCMC <- function(
  X,
  level.VaR = beta,
  levels.RVaR = c(beta1, beta2),
  levels.ES = c(beta2, 1)
){


  N <- dim(X)[1]
  d <- dim(X)[2]
  
  # Expectation
  EXest <- colMeans(X)
  EXse <- apply(X,2,BMest) / sqrt(N)


  # VaR
  VaRest <- ApplyVaR(X,level.VaR)
  VaRse <-  apply((X<VaRest),2,BMest) / sqrt(N) /  cDensEst(X,VaRest)

  # RVaR
  sample.RVaR <- RangeSample(X, levels.RVaR)
  RVaRest <- colMeans(sample.RVaR)
  RVaRse <- apply(sample.RVaR,2,BMest) / sqrt(dim(sample.RVaR)[1])
  
  
  # ES
  sample.ES <- RangeSample(X, levels.ES)
  ESest <- colMeans(sample.ES)
  ESse <- apply(sample.ES,2,BMest) / sqrt(dim(sample.ES)[1])
  

  return(list(
    Meanest = EXest,
    Meanse = EXse,
    RVaRest = RVaRest,
    RVaRse = RVaRse,
    VaRest = VaRest,
    VaRse = VaRse,
    ESest = ESest,
    ESse = ESse
  ))
}




######################################
## Hamiltonian Monte Carlo
######################################


#' @title Leapfrog integrator for Risk Statistics
#' @param   y: Standardized position
#' @param   p: Momentum
#' @param   DU: Gradient of potential energy of Y
#' @param   eps: Stepsize
#' @param   type.riskmeasure: VaR, RVaR or ES
#' @param   type.loss: R or Rplus
#' @param   v: VaR or (VaR1, VaR2)
#' @param   L: Cholesky factor of VaR(X)
#' @return
#'   x: Position
#'   p: Momentum
LeapFrogRiskStats <- function(y, p, DU, eps, type.riskmeasure, type.loss, v, L){

  # flags for the types of risk measures
  flag.RVaR <- (type.riskmeasure == "RVaR")
  flag.Rplus <- (type.loss == "Rplus")
  flag.VaR <- (type.riskmeasure == "VaR")

  d <- length(y) # working dimension
  
  # make a half step for momentum
  p <- p - ( eps / 2 ) * DU(y)  

  # number of constraints
  M <- 1 + flag.RVaR + d * flag.Rplus
  t.seq <- rep(0, M)
  # parameters for the boundaries
  h.mat <- matrix(1, nrow = M, ncol = d)
  v.vec <- rep(0, M)
  v.vec[(M - flag.RVaR) : M] <- v

  if(type.loss == "Rplus") { 
    h.mat[1:d,1:d] = diag(1,d)
  }

  # Rotate the boundaries by L
  # v is unchanged
  h.mat <- h.mat %*% L

  eps.temp <- eps   # temporary epsilon to be updated
  cont <- TRUE 

  while(cont == TRUE) {
       
   y.candidate <- y + eps.temp * p # candidate without reflection

   for(j in 1:M){
      t.seq[j]  <- (v.vec[j] - h.mat[j,] %*% y) / (eps.temp * h.mat[j,] %*% p)
      if( (t.seq[j] < 0.0001) | (t.seq[j] > 1.0001)) {
          t.seq[j] <- Inf
      } # if 0 < t < 1, then the trajectory hits the boundary. Otherwise set Inf
    }

    t.min <- min(t.seq)
    j.min <- which.min(t.seq)

    if(t.min == Inf) {
      y <- y.candidate 
      cont <- FALSE
    } else {
      # Formula for the reflected point
      y.candidate.reflected <- y.candidate - 2 * (y.candidate %*% h.mat[j.min,] - v.vec[j.min]) / (h.mat[j.min,] %*% h.mat[j.min,]) * h.mat[j.min,]
      p.reflected <- ( y.candidate.reflected - y - t.min * eps.temp * p ) / eps.temp / (1 - t.min)
      y <- y + t.min * eps.temp * p 
      p <- p.reflected
      eps.temp <- (1 - t.min) * eps.temp # trajectory proceeded with length t.min * epstemp. 
    }

    }

    p <- p - ( eps / 2 ) * DU(y)  # make the last half step for momentum

  return(list(y = y, p = p)) 
}






#' @title Hamiltonian Monte Carlo for Risk Statistics with standard Gaussian kinetic energy 
#' @param   N: Length of sample path
#' @param   X0: Presample from the target distribution
#' @param   U: Ponential energy
#' @param   DU: Gradient of potential energy
#' @param   type.riskmeasure: Var, RVaR or ES
#' @param   type.loss: R or Rplus
#' @Return
#'      MCMCsample: sample path
#'      ACR: acceptance rate 
#'      H.error: error of Hamiltonians
#'      RunTime: run time
SimulateHMC <- function(N, X0, U, DU, type.riskmeasure, type.loss){

  # set seed 
  set.seed(1221)

  start.time <- Sys.time()


  # dimension (for VaR crisis event, reduce one dimension)
  d <- dim(X0)[2] - (type.riskmeasure == "VaR")

    eps <- 1 * d^{-1/4} # initial stepsize
  Tmax <- 1000 # maximum integration time

  S0 <- rowSums(X0) # presample of the sums

  # restore v
  if (type.riskmeasure == "VaR") {
    v <- mean(S0) # VaR(S)
    X0 <- v * (X0 / rowSums(X0)) # modify the sample
  } else if (type.riskmeasure == "RVaR") {
    v <- c(min(S0), max(S0)) # (VaR1, VaR2)
  } else if (type.riskmeasure == "ES") {
    v <- min(S0) # VaR1
  }

  X0 <- X0[, 1:d] # for VaR crisis event, omit the last column from X0
  Sigma <- var(X0) # Variance of X0
  L <- t(chol(Sigma)) # Cholesky factor s.t. L %*% L^T = Sigma
  L.inv <- solve(L) # Inverse of L
  # X => Y by standardization
  Y0 <- X0 %*% t(L.inv) 

  # Initial value
  y0 <- colMeans(Y0)

  # Modify U and DU for the case of VaR constraint
  # Umod = Potential energy of Y
  # DUmod = Derivative of the potential energy of Y 
  # U = Potential energy of X
  # DU = Derivative of the potential energy of X
  if (type.riskmeasure == "VaR") { 
      Umod <- function(y) {
        x <- c(L %*% y, v - sum(L %*% y)) 
        rslt <- U(x) 
        return(rslt)
      }
      DUmod <- function(y){
        x <- c(L %*% y, v - sum(L %*% y)) 
        rslt <- cbind((t(L)), - t(L) %*% rep(1,d)) %*% DU(x)
        return(as.vector(rslt))
      }
  } else { # risk.measure == "RVaR" or "ES"
      Umod <- function(y) {
        U(as.vector(L %*% y))
      }
      DUmod <- function(y) {
        as.vector(t(L) %*% DU(as.vector(L %*% y)))  
      }
  }


  # Setting of Kinetic energy distributions = Multivariate Standard Normal
  K <- function(p){ 0.5 * t(p) %*% p }
  DK <- function(p) { p }

  N0 <- dim(Y0)[1] # size of the MC presample
  N0 <- min(N0, 2500) # set the threshold

  T.seq <- rep(0,N0)
  # target minimum of alpha
  # converges to 0.65 as d goes to Inf for normal target
  if (d ==1 ){
    alpha.min.target <- 0.98
  } else{
    alpha.min.target <- (0.65 * (d-1) + 1)/d
  }

  cont = TRUE 

  # determine stepsize and integration time based on the MC presample
  while(cont == TRUE){

    alpha.min <- 1

  for(n in 1:N0){ # extend the trajectory from MC presamples
    
    y0 <- Y0[n,]
    y <- y0
    p <- rnorm(d)

    # Euclidean distance from y0
    Edist <- function(x){
      sum((x-y0)^2)^{1/2}
    }

    Zmat <- matrix(0, nrow = (Tmax+1), ncol = 2*d)
    Zmat[1, ] <- c(y, p) # store the trajectory
    H.seq <- rep(0, Tmax+1)  # store the Hamiltonian
    H.current <- Umod(y) + K(p)
    H.seq[1] <- H.current
    dist.diff.seq <- rep(0,Tmax+1)# store the distance

    for (k in 1:Tmax) {
      dist.prev <- Edist(y) # distance from y0
      # extend the trajectory
      leapfrog.rslt <- LeapFrogRiskStats(y, p, DUmod, eps, type.riskmeasure, type.loss, v, L)
      y <- leapfrog.rslt$y
      p <- leapfrog.rslt$p
      Zmat[(k+1), ] <- c(y, p)
      H.seq[k+1] <- Umod(y) + K(p) # new Hamiltonian
      dist.new <- Edist(y) # new distance from y0
      dist.diff.seq[k+1] <- dist.new - dist.prev # increment of the distance 
      # stop the trajectory when the increment changes from >0 to <0
      cond <- (dist.diff.seq[k+1]<0) * (dist.diff.seq[k]>0) 

      if(cond == 1) { break }      

    }

    T.seq[n] <- k # integration time till stopped

    # minimum of the acceptance probabilities 
    alpha.min <- min(c(alpha.min,exp(H.current-H.seq[2:k]) ))

    # if acceptance probability cannot be large enough, stop FOR on n
    if(alpha.min < alpha.min.target){ 
       print(c("stepsize", eps, "not enough small")) # print the number of iteration
       break 
    }


  }  # ENDFOR n

  if(n == N0){ # if all trajectories are generated with acceptance probabilities enough large
    # stop continue
    cont = FALSE
  }else{
    # decrease stepsize and continue
    eps <- eps / 2
    cont = TRUE
  }

  } # while

  # integration time
  T <- as.integer(mean(T.seq))

  ## HMC with eps and T

  Y <- matrix(0, nrow = N + 1, ncol = d)
  H.error.seq <- rep(0, N)
  Y[1,] <- y0  # initial
  count <- 0 # count of acceptance


  for (n in 1:N) {

    y <- Y[n,]
    p <- rnorm(d)
    H.current <- Umod(y) + K(p)

    for (k in 1:T) {
      leapfrog.rslt <- LeapFrogRiskStats(y, p, DUmod, eps, type.riskmeasure, type.loss, v, L)
      y <- leapfrog.rslt$y
      p <- leapfrog.rslt$p
    }
    p <- - p

    H.candidate <- Umod(y) + K(p)

    H.error <- H.candidate - H.current
    H.error.seq[n] <- H.error

    if(is.na(H.error) == TRUE){
      Y[n + 1, ] <- Y[n, ]
      }else if (runif(1) < exp(- H.error)){

      Y[n+1,] <- y
      count <- count + 1

    }else{

      Y[n + 1, ] <- Y[n, ]

    }

    if(n%%10^3 == 0){
      
          print(c("Now", n, "th interation")) # print the number of iteration

      }


  }

  # map back to X
  X <- Y %*% t(L)
  if (type.riskmeasure == "VaR") {
    X <- cbind(X, v- rowSums(X))
  } 

  end.time <- Sys.time()

  run.time <- end.time - start.time 

  return(list(
    MCMCsample = X,
    ACR = count / N,
    H.error = H.error.seq,
    stepsize = eps,
    integration.time = T,
    run.time = run.time
    ))

}

# (1-1) GPD.Cl for RVaR
RVaR.HMC.GPD.Cl <- SimulateHMC(
  N = 10^4, 
  X0 = RVaR.MC.GPD.Cl$MCsample, 
  U = U.GPD.Cl, 
  DU = DU.GPD.Cl, 
  type.riskmeasure = "RVaR", 
  type.loss = "Rplus"
)

# Autocorrelation
acf(RVaR.HMC.GPD.Cl$MCMCsample)
# h = 10 below 0.1

# ACR
RVaR.HMC.GPD.Cl$ACR

# Hamiltonian error
plot(RVaR.HMC.GPD.Cl$H.error, type = "l", xlab="Iteration", ylab="Hamiltonian error")

# Hyperparameters
RVaR.HMC.GPD.Cl$stepsize
RVaR.HMC.GPD.Cl$integration.time

# run time 
RVaR.HMC.GPD.Cl$run.time

# Compare standard errors by BM
BMest(RVaR.MC.GPD.Cl$MCsample) 
BMest(RVaR.HMC.GPD.Cl$MCMCsample)

# Compare statistics
summary(RVaR.HMC.GPD.Cl$MCMCsample)
summary(RVaR.MC.GPD.Cl$MCsample)

# Compare plots
plot3d(RVaR.MC.GPD.Cl$MCsample)
points3d(RVaR.HMC.GPD.Cl$MCMCsample, col = "red")


# (1-2) GPD.Cl for VaR
VaR.HMC.GPD.Cl <- SimulateHMC(
  N = 10^4, 
  X0 = VaR.MC.GPD.Cl$MCsample, 
  U = U.GPD.Cl, 
  DU = DU.GPD.Cl, 
  type.riskmeasure = "VaR", 
  type.loss = "Rplus"
)

# Autocorrelation
acf(VaR.HMC.GPD.Cl$MCMCsample)
# h = 15 below 0.1

# ACR
VaR.HMC.GPD.Cl$ACR

# Hamiltonian error
plot(VaR.HMC.GPD.Cl$H.error, type = "l")

# Hyperparameters
VaR.HMC.GPD.Cl$stepsize
VaR.HMC.GPD.Cl$integration.time

# Run time 
VaR.HMC.GPD.Cl$run.time


# Compare standard errors by BM
BMest(VaR.MC.GPD.Cl$MCsample) 
BMest(VaR.HMC.GPD.Cl$MCMCsample)

# Compare statustucs
summary(VaR.HMC.GPD.Cl$MCMCsample)
summary(VaR.MC.GPD.Cl$MCsample)

# Compare plots
plot3d(VaR.MC.GPD.Cl$MCsample)
points3d(VaR.HMC.GPD.Cl$MCMCsample, col = "red")


# (1-3) pareto.Cl for RVaR
RVaR.HMC.pareto.Cl <- SimulateHMC(
  N = 10^4, 
  X0 = RVaR.MC.pareto.Cl$MCsample, 
  U = U.par.Cl, 
  DU = DU.par.Cl, 
  type.riskmeasure = "RVaR", 
  type.loss = "Rplus"
)


# Autocorrelation
acf(RVaR.HMC.pareto.Cl$MCMCsample)
# h = 21 below 0.1

# ACR
RVaR.HMC.pareto.Cl$ACR

# Hamiltonian error
plot(RVaR.HMC.pareto.Cl$H.error, type = "l", xlab="Iteration", ylab="Hamiltonian error")

# Hyperparameters
RVaR.HMC.pareto.Cl$stepsize
RVaR.HMC.pareto.Cl$integration.time

# Run time 
RVaR.HMC.pareto.Cl$run.time


# Compare standard errors by BM
BMest(RVaR.MC.pareto.Cl$MCsample) 
BMest(RVaR.HMC.pareto.Cl$MCMCsample)

# Compare statistics
summary(RVaR.HMC.pareto.Cl$MCMCsample)
summary(RVaR.MC.pareto.Cl$MCsample)

# Compare plots
plot(RVaR.HMC.pareto.Cl$MCMCsample[1:100,],type="l")
points(RVaR.HMC.pareto.Cl$MCMCsample[1:100,],cex=2)

plot(xdata[1:1500,], xlim = c(0,max.loss[1]), ylim = c(0,max.loss[2]), xlab = "Loss", ylab= "Expense")
points(loss,col="blue")
points(RVaR.HMC.pareto.Cl$MCMCsample[1:10^3,],col="green")
abline(a = v1,b = -1,col="red")
abline(a = v2,b = -1,col="red")
legend("topright", legend  =  c("Data","Simulated sample"),pch = c(1,1), cex = 1.2, pt.cex = 1.2, col = c("black","blue"), bty = "o")


# (1-4) pareto.Cl for VaR
VaR.HMC.pareto.Cl <- SimulateHMC(
  N = 10^4, 
  X0 = VaR.MC.pareto.Cl$MCsample, 
  U = U.par.Cl, 
  DU = DU.par.Cl, 
  type.riskmeasure = "VaR", 
  type.loss = "Rplus"
)


# Autocorrelation
acf(VaR.HMC.pareto.Cl$MCMCsample)
# h = 78 below 0.1

# ACR
VaR.HMC.pareto.Cl$ACR

# Hamiltonian error
plot(VaR.HMC.pareto.Cl$H.error, type = "l", xlab="Iteration", ylab="Hamiltonian error")

# Hyperparameters
VaR.HMC.pareto.Cl$stepsize
VaR.HMC.pareto.Cl$integration.time
# Run time 
VaR.HMC.pareto.Cl$run.time

# Compare standard errors by BM
BMest(VaR.MC.pareto.Cl$MCsample) 
BMest(VaR.HMC.pareto.Cl$MCMCsample)

# Compare statistics
summary(VaR.HMC.pareto.Cl$MCMCsample)
summary(VaR.MC.pareto.Cl$MCsample)

# Compare plots
plot(VaR.HMC.pareto.Cl$MCMCsample[1:100,],type="l")
points(VaR.HMC.pareto.Cl$MCMCsample[1:100,],cex=2)

plot(xdata[1:1500,], xlim = c(0,max.loss[1]), ylim = c(0,max.loss[2]), xlab = "Loss", ylab= "Expense")
points(loss,col="blue")
points(VaR.HMC.pareto.Cl$MCMCsample[1:10^3,],col="green")
abline(a = v1,b = -1,col="red")
abline(a = v2,b = -1,col="red")
legend("topright", legend  =  c("Data","Simulated sample"),pch = c(1,1), cex = 1.2, pt.cex = 1.2, col = c("black","blue"), bty = "o")


# Note: this takes around 1 hour
# (1-3) pareto.Cl for ES
ES.HMC.pareto.Cl <- SimulateHMC(
  N = 10^4, 
  X0 = ES.MC.pareto.Cl$MCsample, 
  U = U.par.Cl, 
  DU = DU.par.Cl, 
  type.riskmeasure = "ES", 
  type.loss = "Rplus",
  Tmax = 1000
)


# Autocorrelation
acf(ES.HMC.pareto.Cl$MCMCsample)

# ACR
ES.HMC.pareto.Cl$ACR
# 0.9958

# Hamiltonian error
plot(ES.HMC.pareto.Cl$H.error, type = "l", xlab="Iteration", ylab="Hamiltonian error")

# Hyperparameters
ES.HMC.pareto.Cl$stepsize
# 5.132424e-05
ES.HMC.pareto.Cl$integration.time
# 838

# Run time 
ES.HMC.pareto.Cl$run.time
# 44.83117 mins

# Compare standard errors by BM
BMest(ES.MC.pareto.Cl$MCsample) 
BMest(ES.HMC.pareto.Cl$MCMCsample)

# Compare statistics
summary(ES.HMC.pareto.Cl$MCMCsample)
summary(ES.MC.pareto.Cl$MCsample)

# Compare plots
plot(ES.HMC.pareto.Cl$MCMCsample[1:100,],type="l")
points(ES.HMC.pareto.Cl$MCMCsample[1:100,],cex=2)

plot(xdata[1:1500,], xlim = c(0,max.loss[1]), ylim = c(0,max.loss[2]), xlab = "Loss", ylab= "Expense")
points(loss,col="blue")
points(ES.HMC.pareto.Cl$MCMCsample[1:400,],col="green")
abline(a = v1,b = -1,col="red")
abline(a = v2,b = -1,col="red")
legend("topright", legend  =  c("Data","Simulated sample"),pch = c(1,1), cex = 1.2, pt.cex = 1.2, col = c("black","blue"), bty = "o")


######################################
## Gibbs Sampler
######################################

#' @title Gibbs sampler for Risk Statistics
#' @param N sample size of Monte Carlo method
#' @param X0: Presample from the target distribution
#' @param pF list of d marginal distribution functions
#' @param qF list of d marginal quantile function
#' @param Copula  "normal", "t", "clayton", "gumbel (d=3 only)" and their survival versions are available for now
#' @param rotate TRUE or FALSE, rotate the copula 180 degree or not
#' @param type.riskmeasure "RVaR" or "ES"
#' @return list of 
#'          MCMCsample  =  MCMC samples of X | S > v
#'          RunTime.MCMC = run time of MCMC
SimulateGS <- function(
  N,
  X0, # MC presample
  pF, # marginal distribution
  qF, # marginal quantile function
  cop.class, #"t" and "clayton" are available for now
  rotate = FALSE, # rotation of copula 
  type.riskmeasure # RVaR or ES
){

  # set seed for experiment
  set.seed(1221)

  start.time <- Sys.time()

  # Set parameters
  d <- dim(X0)[2] # dimension
  N0 <- dim(X0)[1]
  S0 <- rowSums(X0) # presample of the sums

  # restore v
  if (type.riskmeasure == "RVaR") {
    v <- c(min(S0), max(S0)) # (VaR1, VaR2)
    valpha <- v[1]
    vbeta <- v[2]
  } else if (type.riskmeasure == "ES") {
    v <- min(S0) # VaR1
    valpha <- v
    vbeta <- Inf
  }

  Sigma <- var(X0) # Variance of X0
  L <- t(chol(Sigma)) # Cholesky factor s.t. L %*% L^T = Sigma
  L.inv <- solve(L) # Inverse of L
  # X => Y by standardization
  Y0 <- X0 %*% t(L.inv) 

  # Initial value
  x0 <- colMeans(X0)

  # recover the parameters from cop.class
  if(class(cop.class) == "tCopula"){
    
    Pvec <- cop.class@parameters[1:(d*(d-1)/2)]
    nu.C <- cop.class@parameters[(d*(d-1)/2) + 1]

    P <- diag(1,d) # correlation matrix
    for(i in 1:(d-1)){
      for(j in (i+1):d){    
        P[i,j] <- Pvec[i+j-2]
        P[j,i] <- Pvec[i+j-2]
      }
    }  
    Pinv <- solve(P)  # inverse of P

    # inverse of P[-j,-j] for j = 1,...,d
    partialPinv <- vector("list",d)
    for(j in 1:d) { 
      partialPinv[[j]] <- solve(P[-j,-j])
    } 

  } else if (class(cop.class) == "claytonCopula"){

    theta <- cop.class@parameters

  } else {
    print("input t or clayton")
  }

  # define updating function
  if (class(cop.class) == "tCopula") { 

    ## defining the update function
    ##' param cstate current state of the chain
    ##' param j the index of component to be updated
    ##' return an updated vector 
    GSupdate <- function(cstate, j) {

      # result vector
      rslt <- cstate
  
      # transform cstate to t vector
      cstate.t <- qt(capply(cstate, pF), df = nu)

      s1 <- sqrt( 
        (nu + d -1) / (nu + t(cstate.t[-j]) %*% partialPinv[[j]] %*% cstate.t[-j])  
        ) 
      s2 <- t(cstate.t[-j]) %*% Pinv[-j,j] / sqrt(Pinv[j,j])

      Fvalpha <- pt(
        s1 * (s2 + qt(pF[[j]](valpha-sum(cstate[-j])),df = nu) * sqrt(Pinv[j,j])),
        df = nu + d - 1)

      Fvbeta <- pt(
        s1 * (s2 + qt(pF[[j]](vbeta-sum(cstate[-j])),df = nu) * sqrt(Pinv[j,j])),
        df = nu + d - 1)

      Delta <- Fvbeta-Fvalpha


      newy.j <- qF[[j]] ( pt (
        ( qt(Delta * runif(1) + Fvalpha, df = nu + d - 1) - s1 * s2 )  / ( s1 * sqrt(Pinv[j,j])),
        df = nu ) )

      # input the result
      rslt[j] <- newy.j

      return(rslt)

    }

  } else if (class(cop.class) == "claytonCopula" && rotate == FALSE) {

    ## defining the update function
    ##' param cstate current state of the chain
    ##' param j the index of component to be updated
    ##' return an updated vector 
    GSupdate <- function(cstate, j) {

      rslt <- cstate
    
      # marginal transform
      Fcstate.alpha <- capply(c(valpha-sum(cstate[-j]), cstate[-j]), pF)
      Fcstate.beta <- capply(c(vbeta-sum(cstate[-j]), cstate[-j]), pF)

      Fvalpha <- ((1 - d + sum(Fcstate.alpha^{-theta}))/
          (2 - d + sum(Fcstate.alpha[-1]^{-theta}))) ^{1 - d - 1/theta }
      Fvbeta <- ((1 - d + sum(Fcstate.beta^{-theta}))/
          (2 - d + sum(Fcstate.beta[-1]^{-theta}))) ^{1 - d - 1/theta }
      Delta <- Fvbeta-Fvalpha
      # correct?
      newy.j <- qF[[j]]((1 + ((Delta * runif(1) + Fvalpha)^{-1/(d-1+1/theta)}-1)*(2 - d + sum(Fcstate.alpha[-1]^{-theta})))^{-1/theta})
  
      # input the result
      rslt[j] <- newy.j

      return(rslt)

    }

  } else if (class(cop.class) == "claytonCopula" && rotate == TRUE) {

    ## defining the update function
    ##' param cstate current state of the chain
    ##' param j the index of component to be updated
    ##' return an updated vector 
    GSupdate <- function(cstate, j) {

      rslt <- cstate
    
      # 1 - marginal transform
      Fcstate.alpha <- 1 - capply(c(valpha-sum(cstate[-j]), cstate[-j]), pF)
      Fcstate.beta <- 1 - capply(c(vbeta-sum(cstate[-j]), cstate[-j]), pF)


      # Fv in rotated version
      Fvalpha <- 1 - ((1 - d + sum(Fcstate.alpha^{-theta}))/
          (2 - d + sum(Fcstate.alpha[-1]^{-theta}))) ^{1 - d - 1/theta }
      Fvbeta <- 1 - ((1 - d + sum(Fcstate.beta^{-theta}))/
          (2 - d + sum(Fcstate.beta[-1]^{-theta}))) ^{1 - d - 1/theta }
      Delta <- Fvbeta-Fvalpha


      # Inverse in rotated version
      newy.j <- qF[[j]](
        1 - (1 + (( 1-Fvalpha - Delta * runif(1)  )^{-1/(d-1+1/theta)}-1)*(2 - d + sum(Fcstate.alpha[-1]^{-theta})))^{-1/theta}
        )
  
      # input the result
      rslt[j] <- newy.j

      return(rslt)

    }

  }

  ###########################
  # 4. MCMC Estimation
  ############################


  # choice of selection probabilities
 # Mvar <- 100
 # Nmax <- 100
 # if(N0 > Nmax){
 #   X0 <- X0[sample(1:N0,Nmax),]
 #   N0 <- Nmax
  #}

 # var.mat <- matrix(0, nrow = Mvar, ncol = N0)
 # q <- rep(0,d)

 # for(j in 1:d){

  #  for(n in 1:N0){
   #   for(m in 1:Mvar){
    #    var.mat[m,n] <- GSupdate(X0[n, ], j)[j]
     # }
    #}

#    q[j] <- mean(apply(var.mat,2,var))

 # }

  # alternative using Gini formula
#  dist.mat <- matrix(0, nrow = N0, ncol = d)
#  Mtemp <- 50

#  for(j in 1:d){
 #   for(n in 1:N0){
  #    temp <- 0
   #   for(i in 1:Mtemp){
        ##dist.mat[n ,j] <- (GSupdate(X0[n, ], j)[j] - X0[n, j])^2 
    #    temp <- temp + (GSupdate(X0[n, ], j)[j] - X0[n, j])^2 
     # }
     # dist.mat[n ,j] <- temp/Mtemp
   # }
 # }

#  q <- colMeans(dist.mat)

  # Choice of q: Gaussian approximation
  q<- rep(0,d)

  for(j in 1:d){
    q[j] <- Sigma[j,j] - t(Sigma[j,-j]) %*% solve(Sigma[-j,-j]) %*% Sigma[j,-j]
  }

  q <- q/sum(q)

  q

  # d-tuple of indices in each type
  sig.f <- function(x) { sample(1:d, replace = TRUE, prob = q) } 

  # pirate study
  Tmax <- 100
  Npir <- 1000

  acf.target <- 0.15

  Xpir <- matrix(0, nrow = Npir + 1, ncol = d)
  Xpir[1, ] <- x0

  for(n in 1:Npir){

    # current state
    cstate <- Xpir[n,]

    sig.j <- sig.f(1) # this 1 does not matter

    # componentwise update of the chain
    for(j in 1:d) {

      cstate <- GSupdate(cstate, sig.j[j])

    }

    Xpir[n+1,] <- cstate

  }

  acf.rslt <- acf(Xpir,plot=FALSE)$acf

  T <- Tmax

  for(j in 1:d){
    T <- min(T, which.min( acf.rslt[,j,j] > acf.target ))
  }

  print(c("repetition time", T, "selection probability", q)) 

  # storage matrix of the sample path of the Markov chain 
  MCchain <- matrix(0, nrow = N+1, ncol = d)    

  # set initial state 
  MCchain[1, ] <- x0


  for(n in 1:N) {

    # current state
    cstate <- MCchain[n, ]

    sig.j <- sig.f(1) # this 1 does not matter

    # componentwise update of the chain
    for(l in 1:T){
      for(j in 1:d) {
       cstate <- GSupdate(cstate, sig.j[j])    
      }
    }

    MCchain[n+1, ] <- cstate

    # notification of the current iteration 
    if(n%%10^3 == 0){
    
      print(c("the iteration of MCMC finished up to",n)) 

    }

  }


  # MCMC estimator of Expected shortfall
  #ESest.MCMC <- sum(MCMCest)

  # end measuring run time of MCMC 
  end.time <- Sys.time()

  # end measuring run time of MCMC 
  run.time <- end.time - start.time

  return(list(
    MCMCsample = MCchain,
    integration.time = T,
    selection.prob =  q,
    run.time = run.time
    ))
}


ES.GS.GPD.Cl <- SimulateGS(
  N = 10^4,
  X0 = ES.MC.GPD.Cl$MCsample,
  pF = pF.GPD,
  qF = qF.GPD,
  cop.class = cop.class.Cl,
  rotate = TRUE,
  type.riskmeasure = "ES"
)

# Hyperparameters
ES.GS.GPD.Cl$integration.time
ES.GS.GPD.Cl$selection.prob

# Run time
ES.GS.GPD.Cl$run.time

# Autocorrelations
acf(ES.GS.GPD.Cl$MCMCsample)
# h = 15 below 0.1

# Compare standard errors by BM
BMest(ES.MC.GPD.Cl$MCsample) 
BMest(ES.GS.GPD.Cl$MCMCsample)

# Compare statistics
summary(ES.GS.GPD.Cl$MCMCsample)
summary(ES.MC.GPD.Cl$MCsample)

# Compare plots
plot3d(ES.MC.GPD.Cl$MCsample)
points3d(ES.GS.GPD.Cl$MCMCsample, col = "red")


RVaR.GS.t.t <- SimulateGS(
  N = 10^4,
  X0 = RVaR.MC.t.t$MCsample,
  pF = pF.t,
  qF = qF.t,
  cop.class = cop.class.t,
  rotate = FALSE,
  type.riskmeasure = "RVaR"
)

# Hyperparameters
RVaR.GS.t.t$integration.time
RVaR.GS.t.t$selection.prob

# Run time
RVaR.GS.t.t$run.time

# Autocorrelation
acf(RVaR.GS.t.t$MCMCsample)
# h = 10 below 0.1

# Compare standard errors by BM
BMest(RVaR.MC.t.t$MCsample) 
BMest(RVaR.GS.t.t$MCMCsample)

# Compare statistics
summary(RVaR.GS.t.t$MCMCsample)
summary(RVaR.MC.t.t$MCsample)

# Compare plots
plot3d(RVaR.MC.t.t$MCsample)
points3d(RVaR.GS.t.t$MCMCsample, col = "red")


ES.GS.t.t <- SimulateGS(
  N = 10^4,
  X0 = ES.MC.t.t$MCsample,
  pF = pF.t,
  qF = qF.t,
  cop.class = cop.class.t,
  rotate = FALSE,
  type.riskmeasure = "ES"
)

# Hyperparameters
ES.GS.t.t$integration.time
ES.GS.t.t$selection.prob

# Run time
ES.GS.t.t$run.time

# Autocorrelation
acf(ES.GS.t.t$MCMCsample)
# h = 5 below 0.1

# Compare standard errors by BM
BMest(ES.MC.t.t$MCsample) 
BMest(ES.GS.t.t$MCMCsample)

# Compare statistics
summary(ES.GS.t.t$MCMCsample)
summary(ES.MC.t.t$MCsample)

# Compare plots
plot3d(ES.MC.t.t$MCsample)
points3d(ES.GS.t.t$MCMCsample, col = "red")


########################################################
# True allocations for multivaiate t distributions
########################################################

# Levels
level.VaR = beta
levels.RVaR = c(beta1, beta2)
levels.ES = c(beta2, 1)

# standard deviation of the sum
sigma.S <- sqrt(t(rep(1,d)) %*% P %*% rep(1, d))

# True VaE
true.VaR <- sigma.S * qt(level.VaR, df = nu)

# True ES
x.alpha <- qt(levels.ES[1], df = nu)
mu.alpha <- nu / (nu - 1) * dt(x.alpha, df = nu) * (1 + x.alpha^2 / nu)
true.ES <- sigma.S / (1 - levels.ES[1]) * mu.alpha

# True RVaR
x.alpha1 <- qt(levels.RVaR[1], df = nu)
x.alpha2 <- qt(levels.RVaR[2], df = nu)
mu.alpha1 <- nu / (nu - 1) * dt(x.alpha1, df = nu) * (1 + x.alpha1^2 / nu)
mu.alpha2 <- nu / (nu - 1) * dt(x.alpha2, df = nu) * (1 + x.alpha2^2 / nu)
true.RVaR <- sigma.S / (levels.RVaR[2] - levels.RVaR[1]) * (mu.alpha1 - mu.alpha2)


#' @title Compute the true Risk contribution for elliptical distributions
#' @param P correlation matrix
#' @param sigvec vector of standard deviations
#' @param v given risk measure of the sum
#' @return risk contributions
AllocateElliptical <- function (P, sigvec, v) {
  Sigmat <- sigvec %*% t(sigvec) * P 
  ratio <- rowSums(Sigmat)
  # true allocation given the Expected shortfall
  rslt <- ratio / sum(ratio) * v
  
  return(rslt)

}

# True allocations
true.VaR.cont <- AllocateElliptical(P, rep(1,d), true.VaR)
true.RVaR.cont <- AllocateElliptical(P, rep(1,d), true.RVaR)
true.ES.cont <- AllocateElliptical(P, rep(1,d), true.ES)

# Compare with the estimates
#colMeans(RVaR.GS.t.t$MCMCsample)
#colMeans(ES.GS.t.t$MCMCsample)


##################################################################
### Summary and Outputs for Simulation Study
##################################################################

# crisis event
# VaR level = 0.99
# RVaR levels = (0.975,0.99)
# ES level = 0.99

# marginal risk measures
# VaR level = 0.99
# RVaR levels = (0.975,0.99)
# ES level = 0.99

# Sample size
# N_MC = 10^4
# N_MCMC = 10^4

# (1) GPD-Cl-VaR => HMC
VaR.HMC.GPD.Cl$ACR
# ACR = 0.9957
VaR.HMC.GPD.Cl$stepsize
# stepsize = 0.2102241
VaR.HMC.GPD.Cl$integration.time
# integration time =12
VaR.HMC.GPD.Cl$run.time
# run time =  1.27718 mins


# (2) GPD-Cl-RVaR => HMC
RVaR.HMC.GPD.Cl$ACR
# ACR = 0.9839
RVaR.HMC.GPD.Cl$stepsize
# stepsize = 0.09497946
RVaR.HMC.GPD.Cl$integration.time
# integration time = 13
RVaR.HMC.GPD.Cl$run.time
# run time = 1.649254 mins



# (3) GPD-Cl-ES => GS
# ACR = 1
ES.GS.GPD.Cl$selection.prob
# selection prob = 0.2212948 0.3620043 0.4167009
ES.GS.GPD.Cl$integration.time
# integration time = 12
ES.GS.GPD.Cl$run.time
# run time =  1.79828 mins


# (4) t-t-RVaR => GS
# ACR = 1
RVaR.GS.t.t$selection.prob
# selection prob = 0.3305423 0.3487557 0.3207020
RVaR.GS.t.t$integration.time
# integration time = 10
RVaR.GS.t.t$run.time
# run time = 56.98274 secs


# (5) t-t-ES => GS
# ACR = 1
ES.GS.t.t$selection.prob
# selection prob =  0.2410283 0.5038347 0.2551370
ES.GS.t.t$integration.time
# integration time = 4
ES.GS.t.t$run.time
# run time =  22.40807 secs


# (1) and (2), Hamiltonian error
Spar <- par(pty = "s")
par(mfcol=c(1,2),Spar)
plot(VaR.HMC.GPD.Cl$H.error[-5826], type = "l", xlab="Iteration", ylab="Hamiltonian error", ylim = c(-0.5,0.5), main = "VaR crisis event")
# 5826: outlier?
plot(RVaR.HMC.GPD.Cl$H.error, type = "l", xlab="Iteration", ylab="Hamiltonian error", ylim = c(-0.5,0.5), main = "RVaR crisis event")

########################################
## Tables for Simulation Study
########################################


## Table 1
VaR.GPD.Cl.table <- cbind(
t(matrix(unlist(RiskStats.MC(VaR.MC.GPD.Cl$MCsample)),nrow=3)),
t(matrix(unlist(RiskStats.MCMC(VaR.HMC.GPD.Cl$MCMCsample)),nrow=3))
)
rownames(VaR.GPD.Cl.table) <- c("Mean","MeanSE","RVaR","RVaRSE","VaR","VaRSE","ES","ESSE")

RVaR.GPD.Cl.table <- cbind(
t(matrix(unlist(RiskStats.MC(RVaR.MC.GPD.Cl$MCsample)),nrow=3)),
t(matrix(unlist(RiskStats.MCMC(RVaR.HMC.GPD.Cl$MCMCsample)),nrow=3))
)
rownames(RVaR.GPD.Cl.table) <- c("Mean","MeanSE","RVaR","RVaRSE","VaR","VaRSE","ES","ESSE")

xtable(VaR.GPD.Cl.table,digit= 3)
xtable(RVaR.GPD.Cl.table,digit= 3)


## Table 2
ES.GPD.Cl.table <- cbind(
t(matrix(unlist(RiskStats.MC(ES.MC.GPD.Cl$MCsample)),nrow=3)),
t(matrix(unlist(RiskStats.MCMC(ES.GS.GPD.Cl$MCMCsample)),nrow=3))
)
rownames(ES.GPD.Cl.table) <- c("Mean","MeanSE","RVaR","RVaRSE","VaR","VaRSE","ES","ESSE")

RVaR.t.t.table <- cbind(
t(matrix(unlist(RiskStats.MC(RVaR.MC.t.t$MCsample)),nrow=3)),
t(matrix(unlist(RiskStats.MCMC(RVaR.GS.t.t$MCMCsample)),nrow=3))
)
rownames(RVaR.t.t.table) <- c("Mean","MeanSE","RVaR","RVaRSE","VaR","VaRSE","ES","ESSE")

ES.t.t.table <- cbind(
t(matrix(unlist(RiskStats.MC(ES.MC.t.t$MCsample)),nrow=3)),
t(matrix(unlist(RiskStats.MCMC(ES.GS.t.t$MCMCsample)),nrow=3))
)
rownames(ES.t.t.table) <- c("Mean","MeanSE","RVaR","RVaRSE","VaR","VaRSE","ES","ESSE")

# Bias of RVaR contributions
xtable(rbind(rep(1,6),
RVaR.t.t.table[1,] - rep(true.RVaR.cont, 2)),
digit=3)

# Bias of ES contributions
xtable(rbind(rep(1,6),
ES.t.t.table[1,] - rep(true.ES.cont, 2)),
digit=3
)

xtable(ES.GPD.Cl.table,digit= 3)
xtable(RVaR.t.t.table,digit= 3)
xtable(ES.t.t.table,digit= 3)


#####################################################
### Summary and Outputs for Empirical Study
#####################################################

# crisis event
# VaR level = 0.99
# RVaR levels = (0.975,0.99)
# ES level = 0.99

# marginal risk measures
# VaR level = 0.99
# RVaR levels = (0.975,0.99)
# ES level = 0.99

# Sample size
# N_MC = 10^5
# N_MCMC = 10^4

# (1) GPD-Cl-VaR => HMC
VaR.HMC.pareto.Cl$ACR
# ACR = 0.9972
VaR.HMC.pareto.Cl$stepsize
# stepsize = 0.015625
VaR.HMC.pareto.Cl$integration.time
# integration time = 34
VaR.HMC.pareto.Cl$run.time
# run time = 2.007789 mins

# (2) GPD-Cl-VaR => HMC
RVaR.HMC.pareto.Cl$ACR
# ACR = 0.9865
RVaR.HMC.pareto.Cl$stepsize
# stepsize = 0.02627801
RVaR.HMC.pareto.Cl$integration.time
# integration time = 39
RVaR.HMC.pareto.Cl$run.time
# run time = 2.689039 mins


# (3) GPD-Cl-VaR => HMC
ES.HMC.pareto.Cl$ACR
# ACR = 0.9958
ES.HMC.pareto.Cl$stepsize
# stepsize = 5.132424e-05
ES.HMC.pareto.Cl$integration.time
# integration time = 838
ES.HMC.pareto.Cl$run.time
# run time = 44.83117 mins

# Hamiltonian errors
Spar <- par(pty = "s")
par(mfcol=c(1,3),ps = 13,mar = c(4,4,4,1),mgp = c(2.5,1,0), Spar)
plot(VaR.HMC.pareto.Cl$H.error, type = "l", xlab="Iteration", ylab="Hamiltonian error",ylim=c(-0.7,0.3), main = "VaR crisis event")
plot(RVaR.HMC.pareto.Cl$H.error, type = "l", xlab="Iteration", ylab="Hamiltonian error",ylim=c(-0.7,0.3), main = "RVaR crisis event")
plot(ES.HMC.pareto.Cl$H.error, type = "l", xlab="Iteration", ylab="Hamiltonian error",ylim=c(-0.7,0.3), main = "ES crisis event")


# Figure of MCMC plots
par(mfcol=c(1,3),ps = 13,mar = c(4,4,4,1),mgp = c(2.5,1,0), Spar)

#par(mfcol=c(1,3),Spar)

#plot(VaR.HMC.pareto.Cl$MCMCsample[1:1500,], xlim = c(0,1.5*max.loss[1]), ylim = c(0,1.5*max.loss[2]), xlab = "Loss", ylab= "Expense",col="green",type="n",main = "(a) Loss distribution")
#points(loss,col="blue")
#points(xdata[1:1500,],col="black")
#abline(a = v1,b = -1,col="red")
#abline(a = v2,b = -1,col="red")
#legend("topright", legend  =  c("Data","MC sample"),pch = c(1,1), cex = 1.2, pt.cex = 1.2, col = c("black","blue"), bty = "o",horiz = TRUE)

plot(VaR.HMC.pareto.Cl$MCMCsample[1:1500,], xlim = c(0,1.5*max.loss[1]), ylim = c(0,1.5*max.loss[2]), 
  xlab = "Loss", ylab= "Expense",col="green",main = "VaR crisis event",cex.lab=1.5,cex.main =2)
points(loss,col="blue")
points(xdata[1:1500,],col="black")
abline(a = v1,b = -1,col="red")
abline(a = v2,b = -1,col="red")
#legend("topright", legend  =  c("MCMC sample"),pch = c(1,1), cex = 1.2, pt.cex = 1.2, col = c("green"), bty = "o")


plot(RVaR.HMC.pareto.Cl$MCMCsample[1:1500,], xlim = c(0,1.5*max.loss[1]), ylim = c(0,1.5*max.loss[2]), 
  xlab = "Loss", ylab= "Expense",col="green",main = "RVaR crisis event",cex.lab=1.5,cex.main =2)
points(loss,col="blue")
points(xdata[1:1500,],col="black")
abline(a = v1,b = -1,col="red")
abline(a = v2,b = -1,col="red")
#legend("topright", legend  =  c("MCMC sample"),pch = c(1,1), cex = 1.2, pt.cex = 1.2, col = c("green"), bty = "o")

plot(ES.HMC.pareto.Cl$MCMCsample[1:1500,], xlim = c(0,1.5*max.loss[1]), ylim = c(0,1.5*max.loss[2]),
 xlab = "Loss", ylab= "Expense",col="green",main = "ES crisis event",cex.lab=1.5,cex.main =2)
points(loss,col="blue")
points(xdata[1:1500,],col="black")
abline(a = v1,b = -1,col="red")
abline(a = v2,b = -1,col="red")
#legend("topright", legend  =  c("MCMC sample"),pch = c(1,1), cex = 1.2, pt.cex = 1.2, col = c("green"), bty = "o")



## Tables
VaR.pareto.Cl.table <- cbind(
t(matrix(unlist(RiskStats.MC(VaR.MC.pareto.Cl$MCsample)),nrow=2)),
t(matrix(unlist(RiskStats.MCMC(VaR.HMC.pareto.Cl$MCMCsample)),nrow=2))
)
rownames(VaR.pareto.Cl.table) <- c("Mean","MeanSE","RVaR","RVaRSE","VaR","VaRSE","ES","ESSE")

RVaR.pareto.Cl.table <- cbind(
t(matrix(unlist(RiskStats.MC(RVaR.MC.pareto.Cl$MCsample)),nrow=2)),
t(matrix(unlist(RiskStats.MCMC(RVaR.HMC.pareto.Cl$MCMCsample)),nrow=2))
)
rownames(RVaR.pareto.Cl.table) <- c("Mean","MeanSE","RVaR","RVaRSE","VaR","VaRSE","ES","ESSE")

ES.pareto.Cl.table <- cbind(
t(matrix(unlist(RiskStats.MC(ES.MC.pareto.Cl$MCsample)),nrow=2)),
t(matrix(unlist(RiskStats.MCMC(ES.HMC.pareto.Cl$MCMCsample)),nrow=2))
)
rownames(ES.pareto.Cl.table) <- c("Mean","MeanSE","RVaR","RVaRSE","VaR","VaRSE","ES","ESSE")


xtable(VaR.pareto.Cl.table,digit= 3)
xtable(RVaR.pareto.Cl.table,digit= 3)
xtable(ES.pareto.Cl.table,digit= 3)

###########################################
# Detailed Comparison 
###########################################

#' @title Comparison of MC and MCMC estimators under multivariate t distribution
#' @param N.MC sample size of Monte Carlo method
#' @param N.MC sample size of MCMC method
#' @param d dimension
#' @param dmax maximum possible dimension
#' @param levels confidence levels
#' @param method.MCMC "HMC" or "GS"
#' @param eps.HMC stepsize of HMC
#' @param T.HMC integration time of HMC
#' @return list of 
#'          MCMCsample  =  MCMC samples of X | S > v
#'          RunTime.MCMC = run time of MCMC
CompareMCMC = function(
  N.MC = 10^4,
  N.MCMC = 10^4,
  d, # dimension
  d.max = 12,
  levels,
  method.MCMC,
  eps.HMC = NA,
  T.HMC = NA
){

## type of risk measures
if(length(levels) == 1){ type.riskmeasure <- "VaR" }
if(length(levels) == 2){ 
  if(levels[2] == 1){ type.riskmeasure <- "ES" 
    }else{ type.riskmeasure <- "RVaR"  }
}

### loss model = multivariate student t

## Marginal distributions
nu = 6
pF <- rep(list(function(x) { pt(x, df = nu) }), d) # cdfs of student's t
qF <- rep(list(function(q) { qt(q, df = nu) }), d) # quantiles of student's t
dF <- rep(list(function(x) { dt(x, df = nu) }), d) # pdfs of student's t

## t Copula
P <- equicorr(d, 1/dmax)
P.inv <- solve(P)  # inverse of P
Pvec <- P2p(P)
cop.class <- ellipCopula("t", dim = d, param = Pvec, dispstr = "un", df = nu)


###################################################
## MC estimation
###################################################
# buffered so that at least 100 samples are guaranteed
SimulateMC <- function(N, Copula, qF, levels) {

  delta <- 0 # modification of the level of VaR crisis event (changed)

  set.seed(1221)  # Set seed

  start.time <- Sys.time()

  # simulate copula
  udata <- 1-rCopula(N, Copula) 
  # Note: I rotate for Clayton. Rotation does not matter for t copula.

  xdata <- capply(udata, qF)
  sdata <- rowSums(xdata)

  # MC samples in {S = VaR1}
  if(length(levels) == 1){
    delta <- 0.01
    levels.L <- max(0,levels - delta) #(changed)
    levels.U <- min(1,levels + delta) #(changed)

    # Note: I take samples falling in the 0.1% window around VaR1
    valpha <- sort(sdata)[N * levels]
    valpha.L <- sort(sdata)[N * levels.L]
    valpha.U <- sort(sdata)[N * levels.U]
    MCsample <- xdata[( valpha.L < sdata ) & (sdata <= valpha.U),]
    MCsample <- MCsample/rowSums(MCsample) * valpha

    # VaR estimate
    valpha1 <- valpha
    valpha2 <- valpha
  }

  # MC samples in {VaR1 < S < VaR2}
  if(length(levels)==2){

    #modification
    if(N*(levels[2]-levels[1])<=100){
      levels[1] <- levels[1]-0.01
    }
    valpha.L <- sort(sdata)[N * levels[1]]
    valpha.U <- sort(sdata)[N * levels[2]]
    MCsample <- xdata[( valpha.L < sdata ) & (sdata <= valpha.U),]

    # VaR estimates
    valpha1 <- valpha.L
    valpha2 <- valpha.U

  }

  end.time <- Sys.time()
  runtime <- end.time - start.time

  return(list(
    MCsample = MCsample,
    VaRests = c(valpha1,valpha2),
    runtime = runtime
    ))
} 

# MC estimation

start.time.MC <- Sys.time()
MC.sim <- SimulateMC(N = N.MC, Copula = cop.class, qF = qF, levels = levels)
MCsample <- MC.sim$MCsample
MC.rslt <- RiskStats.MC(MCsample)
MCest <- MC.rslt$Meanest 
MCse <- MC.rslt$Meanse
end.time.MC <- Sys.time()
runtime.MC <- end.time.MC - start.time.MC

trueAC <- AllocateElliptical(P,rep(1,d),sum(MCest))

###################################################
## MCMC estimation
###################################################
if(method.MCMC == "GS"){
  N <- N.MCMC
  X0 <- MC.sim$MCsample
  # set seed for experiment
  set.seed(1221)

  # Set parameters
  d <- dim(X0)[2] # dimension
  N0 <- dim(X0)[1]
  S0 <- rowSums(X0) # presample of the sums

  # restore v
  if (type.riskmeasure == "RVaR") {
    v <- c(min(S0), max(S0)) # (VaR1, VaR2)
    valpha <- v[1]
    vbeta <- v[2]
  } else if (type.riskmeasure == "ES") {
    v <- min(S0) # VaR1
    valpha <- v
    vbeta <- Inf
  }

  # Initial value
  x0 <- colMeans(X0)

  # parameters
  nu.C <- nu
  Pinv <- P.inv
  # inverse of P[-j,-j] for j = 1,...,d
  partialPinv <- vector("list",d)
  for(j in 1:d) { 
    partialPinv[[j]] <- solve(P[-j,-j])
  } 

  # define updating function
 
  ## defining the update function
  ##' param cstate current state of the chain
  ##' param j the index of component to be updated
  ##' return an updated vector 
  GSupdate <- function(cstate, j) {

    # result vector
    rslt <- cstate
  
    # transform cstate to t vector
    cstate.t <- qt(capply(cstate, pF), df = nu)

    s1 <- sqrt( 
      (nu + d -1) / (nu + t(cstate.t[-j]) %*% partialPinv[[j]] %*% cstate.t[-j])  
      ) 
    s2 <- t(cstate.t[-j]) %*% Pinv[-j,j] / sqrt(Pinv[j,j])

    Fvalpha <- pt(
      s1 * (s2 + qt(pF[[j]](valpha-sum(cstate[-j])),df = nu) * sqrt(Pinv[j,j])),
      df = nu + d - 1)

    Fvbeta <- pt(
        s1 * (s2 + qt(pF[[j]](vbeta-sum(cstate[-j])),df = nu) * sqrt(Pinv[j,j])),
        df = nu + d - 1)

    Delta <- Fvbeta-Fvalpha


    newy.j <- qF[[j]] ( pt (
        ( qt(Delta * runif(1) + Fvalpha, df = nu + d - 1) - s1 * s2 )  / ( s1 * sqrt(Pinv[j,j])),
        df = nu ) )

    # input the result
    rslt[j] <- newy.j

    return(rslt)

    }

  ###########################
  # 4. MCMC Estimation
  ############################

  # Choice of q: Gaussian approximation
  q <- rep(1/d,d)


  # d-tuple of indices in each type
  sig.f <- function(x) { sample(1:d, replace = TRUE, prob = q) } 

  # pirate study
  Tmax <- 100
  Npir <- 1000

  acf.target <- 0.15

  Xpir <- matrix(0, nrow = Npir + 1, ncol = d)
  Xpir[1, ] <- x0

  for(n in 1:Npir){

    # current state
    cstate <- Xpir[n,]

    sig.j <- sig.f(1) # this 1 does not matter

    # componentwise update of the chain
    for(j in 1:d) {

      cstate <- GSupdate(cstate, sig.j[j])

    }

    Xpir[n+1,] <- cstate

  }

  acf.rslt <- acf(Xpir,plot=FALSE)$acf

  T <- Tmax

  for(j in 1:d){
    T <- min(T, which.min( acf.rslt[,j,j] > acf.target ))
  }

  print(c("repetition time", T, "selection probability", q)) 

  # storage matrix of the sample path of the Markov chain 
  MCchain <- matrix(0, nrow = N+1, ncol = d)    

  # set initial state 
  MCchain[1, ] <- x0

  start.time.MCMC <- Sys.time()
  for(n in 1:N) {

    # current state
    cstate <- MCchain[n, ]

    sig.j <- sig.f(1) # this 1 does not matter

    # componentwise update of the chain
    for(l in 1:T){
      for(j in 1:d) {
       cstate <- GSupdate(cstate, sig.j[j])    
      }
    }

    MCchain[n+1, ] <- cstate

    # notification of the current iteration 
    if(n%%10^3 == 0){
    
      print(c("the iteration of MCMC finished up to",n)) 

    }

  }
  end.time.MCMC <- Sys.time()
  # end measuring run time of MCMC 
  run.time.MCMC <- end.time.MCMC - start.time.MCMC

  MCMCsample <- MCchain
  integration.time <- T
  selection.prob <-  q
  runtime.MCMC <- run.time.MCMC

  MCMCrslt <- RiskStats.MCMC(MCMCsample)
  MCMCest <- MCMCrslt$Meanest
  MCMCse <- MCMCrslt$Meanse

  eps = NA
 }

  if(method.MCMC == "HMC"){

  if(is.na(eps.HMC) || is.na(T.HMC)){
    HMC.opt <- TRUE  
  }else{
    HMC.opt <- FALSE
  }
 
  U = function(x){
    as.numeric(0.5 * (nu+d) * log(1+ (t(x) %*% P.inv %*% x)/nu))
  }

  DU = function(x){
    as.vector((nu + d)* (P.inv %*% x ) / as.numeric(nu + t(x) %*% P.inv %*%x))
  }

  # Kinetic energy distributions
  K <- function(p){ 0.5 * t(p) %*% p } # Kinetic energy
  DK <- function(p) { p } 

  type.loss <- "R"

  # set seed 
  set.seed(1221)

  N <- N.MCMC
  X0 <- MC.sim$MCsample
  
  # dimension (for VaR crisis event, reduce one dimension)
  d <- dim(X0)[2] - (type.riskmeasure == "VaR")

  eps <- 1 * d^{-1/4} # initial stepsize
  Tmax <- 1000 # maximum integration time

  S0 <- rowSums(X0) # presample of the sums

  # restore v
  if (type.riskmeasure == "VaR") {
    v <- mean(S0) # VaR(S)
    X0 <- v * (X0 / rowSums(X0)) # modify the sample
  } else if (type.riskmeasure == "RVaR") {
    v <- c(min(S0), max(S0)) # (VaR1, VaR2)
  } else if (type.riskmeasure == "ES") {
    v <- min(S0) # VaR1
  }

  X0 <- X0[, 1:d] # for VaR crisis event, omit the last column from X0
  Sigma <- var(X0) # Variance of X0
  L <- t(chol(Sigma)) # Cholesky factor s.t. L %*% L^T = Sigma
  L.inv <- solve(L) # Inverse of L
  # X => Y by standardization
  Y0 <- X0 %*% t(L.inv) 

  # Initial value
  y0 <- colMeans(Y0)

  # Modify U and DU for the case of VaR constraint
  # Umod = Potential energy of Y
  # DUmod = Derivative of the potential energy of Y 
  # U = Potential energy of X
  # DU = Derivative of the potential energy of X
  if (type.riskmeasure == "VaR") { 
      Umod <- function(y) {
        x <- c(L %*% y, v - sum(L %*% y)) 
        rslt <- U(x) 
        return(rslt)
      }
      DUmod <- function(y){
        x <- c(L %*% y, v - sum(L %*% y)) 
        rslt <- cbind((t(L)), - t(L) %*% rep(1,d)) %*% DU(x)
        return(as.vector(rslt))
      }
  } else { # risk.measure == "RVaR" or "ES"
      Umod <- function(y) {
        U(as.vector(L %*% y))
      }
      DUmod <- function(y) {
        as.vector(t(L) %*% DU(as.vector(L %*% y)))  
      }
  }

  N0 <- dim(Y0)[1] # size of the MC presample
  N0 <- min(N0, 2500) # set the threshold

  if(HMC.opt == FALSE){
    eps <- eps.HMC
    T <- T.HMC
  }else{

  T.seq <- rep(0,N0)
  # target minimum of alpha
  # converges to 0.65 as d goes to Inf for normal target
  if (d ==1 ){
    alpha.min.target <- 0.98
  } else{
    alpha.min.target <- (0.65 * (d-1) + 1)/d
  }

  cont = TRUE 

  # determine stepsize and integration time based on the MC presample
  while(cont == TRUE){

    alpha.min <- 1

  for(n in 1:N0){ # extend the trajectory from MC presamples
    
    y0 <- Y0[n,]
    y <- y0
    p <- rnorm(d)

    # Euclidean distance from y0
    Edist <- function(x){
      sum((x-y0)^2)^{1/2}
    }

    Zmat <- matrix(0, nrow = (Tmax+1), ncol = 2*d)
    Zmat[1, ] <- c(y, p) # store the trajectory
    H.seq <- rep(0, Tmax+1)  # store the Hamiltonian
    H.current <- Umod(y) + K(p)
    H.seq[1] <- H.current
    dist.diff.seq <- rep(0,Tmax+1)# store the distance

    for (k in 1:Tmax) {
      dist.prev <- Edist(y) # distance from y0
      # extend the trajectory
      leapfrog.rslt <- LeapFrogRiskStats(y, p, DUmod, eps, type.riskmeasure, type.loss, v, L)
      y <- leapfrog.rslt$y
      p <- leapfrog.rslt$p
      Zmat[(k+1), ] <- c(y, p)
      H.seq[k+1] <- Umod(y) + K(p) # new Hamiltonian
      dist.new <- Edist(y) # new distance from y0
      dist.diff.seq[k+1] <- dist.new - dist.prev # increment of the distance 
      # stop the trajectory when the increment changes from >0 to <0
      cond <- (dist.diff.seq[k+1]<0) * (dist.diff.seq[k]>0) 

      if(cond == 1) { break }      

    }

    T.seq[n] <- k # integration time till stopped

    # minimum of the acceptance probabilities 
    alpha.min <- min(c(alpha.min,exp(H.current-H.seq[2:k]) ))

    # if acceptance probability cannot be large enough, stop FOR on n
    if(alpha.min < alpha.min.target){ 
       print(c("stepsize", eps, "not enough small")) # print the number of iteration
       break 
    }


  }  # ENDFOR n

  if(n == N0){ # if all trajectories are generated with acceptance probabilities enough large
    # stop continue
    cont = FALSE
  }else{
    # decrease stepsize and continue
    eps <- eps / 2
    cont = TRUE
  }

  } # while

  # integration time
  T <- as.integer(mean(T.seq))

  }


  ## HMC with eps and T

  Y <- matrix(0, nrow = N + 1, ncol = d)
  H.error.seq <- rep(0, N)
  Y[1,] <- y0  # initial
  count <- 0 # count of acceptance

  start.time.MCMC <- Sys.time()

  for (n in 1:N) {

    y <- Y[n,]
    p <- rnorm(d)
    H.current <- Umod(y) + K(p)

    for (k in 1:T) {
      leapfrog.rslt <- LeapFrogRiskStats(y, p, DUmod, eps, type.riskmeasure, type.loss, v, L)
      y <- leapfrog.rslt$y
      p <- leapfrog.rslt$p
    }
    p <- - p

    H.candidate <- Umod(y) + K(p)

    H.error <- H.candidate - H.current
    H.error.seq[n] <- H.error

    if(is.na(H.error) == TRUE){
      Y[n + 1, ] <- Y[n, ]
      }else if (runif(1) < exp(- H.error)){

      Y[n+1,] <- y
      count <- count + 1

    }else{

      Y[n + 1, ] <- Y[n, ]

    }

    if(n%%10^3 == 0){
      
          print(c("Now", n, "th interation")) # print the number of iteration

      }

  }

  end.time.MCMC <- Sys.time()

  runtime.MCMC <- end.time.MCMC - start.time.MCMC

  # map back to X
  X <- Y %*% t(L)
  if (type.riskmeasure == "VaR") {
    d <- d + 1
    XX <- matrix(0,ncol=d, nrow=N.MCMC)
    # elliminate the effect of choosing the last variable to be excluded
    Nvec <- floor(seq(1,10^4,length=d+1))
    for(j in 1:d){
      XX[Nvec[j]:Nvec[j+1],-j] <- X[Nvec[j]:Nvec[j+1],]
      XX[Nvec[j]:Nvec[j+1],j] <- v- rowSums(X[Nvec[j]:Nvec[j+1],])
      }
  }else{
    XX <- X
  }
  

  MCMCsample = XX
    runtime.MCMC = runtime.MCMC

  MCMCrslt <- RiskStats.MCMC(MCMCsample)
  MCMCest <- MCMCrslt$Meanest
  MCMCse <- MCMCrslt$Meanse

  }


###################################################
## Comparison
###################################################

if(attr(runtime.MCMC, "unit") == "mins"){
  runtime.MCMC <- as.numeric(runtime.MCMC) * 60
}
if(attr(runtime.MC, "unit") == "mins"){
  runtime.MC <- as.numeric(runtime.MC) * 60
}


bias.MC <- MCest - trueAC
bias.MCMC <- MCMCest - trueAC
SSE.MC <- (MCse^2) * as.numeric(runtime.MC) / as.numeric(runtime.MCMC)
SSE.MCMC <- (MCMCse^2)

MSE.MC <- bias.MC^2 + SSE.MC
MSE.MCMC <- bias.MCMC^2 + SSE.MCMC

  return(list(
    MCsample = MCsample,
    MCest = MCest,
    MCse = MCse,
    runtime.MC = runtime.MC,
    bias.MC = bias.MC,
    SSE.MC = SSE.MC,
    MSE.MC = MSE.MC,
    MCMCsample = MCMCsample,
    ACR = count / N.MCMC,
    H.error = H.error.seq,
    MCMCest = MCMCest,
    MCMCse = MCMCse,
    runtime.MCMC = runtime.MCMC,
    bias.MCMC = bias.MCMC,
    SSE.MCMC = SSE.MCMC,
    MSE.MCMC = MSE.MCMC,
    trueAC = trueAC,
    stepsize = eps,
    integration.time = T
    ))

}




##################################################################
### Comparison of MC and MCMC
##################################################################

dmax <- 12 # max dimension considered 
dset <- c(4,6,8,10)  # set of dimensions

# set of confidence levels
levelset.VaR <- c(0.9,0.99,0.999,0.9999)
levelset.RVaR1 <- c(0.9,0.9,0.99,0.999)
levelset.RVaR2 <- c(0.9999,0.99,0.999,0.9999)
levelset.ES <- c(0.9,0.99,0.999,0.9999)


##################################################################
### Comparison for differnet dimensions
##################################################################

# VaR
HMC4.VaR = CompareMCMC(d=4,levels = 0.99,method.MCMC="HMC")
HMC6.VaR = CompareMCMC(d=6,levels = 0.99,method.MCMC="HMC")
HMC8.VaR = CompareMCMC(d=8,levels = 0.99,method.MCMC="HMC")
HMC10.VaR = CompareMCMC(d=10,levels = 0.99,method.MCMC="HMC")

# Hyperparameters
HMC4.VaR$stepsize
HMC6.VaR$stepsize
HMC8.VaR$stepsize
HMC10.VaR$stepsize
HMC4.VaR$integration.time
HMC6.VaR$integration.time
HMC8.VaR$integration.time
HMC10.VaR$integration.time

# Run times
HMC4.VaR$runtime.MC
HMC4.VaR$runtime.MCMC
HMC6.VaR$runtime.MC
HMC6.VaR$runtime.MCMC
HMC8.VaR$runtime.MC
HMC8.VaR$runtime.MCMC
HMC10.VaR$runtime.MC
HMC10.VaR$runtime.MCMC

# RVaR
HMC4.RVaR = CompareMCMC(d=4,levels = c(0.95,0.99),method.MCMC="HMC")
HMC6.RVaR = CompareMCMC(d=6,levels = c(0.95,0.99),method.MCMC="HMC")
HMC8.RVaR = CompareMCMC(d=8,levels = c(0.95,0.99),method.MCMC="HMC")
HMC10.RVaR = CompareMCMC(d=10,levels = c(0.95,0.99),method.MCMC="HMC")
GS4.RVaR = CompareMCMC(d=4,levels = c(0.95,0.99),method.MCMC="GS")
GS6.RVaR = CompareMCMC(d=6,levels = c(0.95,0.99),method.MCMC="GS")
GS8.RVaR = CompareMCMC(d=8,levels = c(0.95,0.99),method.MCMC="GS")
GS10.RVaR = CompareMCMC(d=10,levels = c(0.95,0.99),method.MCMC="GS")

#ES
HMC4.ES = CompareMCMC(d=4,levels = c(0.99,1),method.MCMC="HMC")
HMC6.ES = CompareMCMC(d=6,levels = c(0.99,1),method.MCMC="HMC")
HMC8.ES = CompareMCMC(d=8,levels = c(0.99,1),method.MCMC="HMC")
HMC10.ES = CompareMCMC(d=10,levels = c(0.99,1),method.MCMC="HMC")
GS4.ES = CompareMCMC(d=4,levels = c(0.99,1),method.MCMC="GS")
GS6.ES = CompareMCMC(d=6,levels = c(0.99,1),method.MCMC="GS")
GS8.ES = CompareMCMC(d=8,levels = c(0.99,1),method.MCMC="GS")
GS10.ES = CompareMCMC(d=10,levels = c(0.99,1),method.MCMC="GS")


# Boxplot

par(mfrow=c(3,3), mar = c(7, 5, 4, 2),mgp = c(3.5, 1, 0))

# Bias
boxplot(
HMC4.VaR$bias.MC,
HMC4.VaR$bias.MCMC,
HMC6.VaR$bias.MC,
HMC6.VaR$bias.MCMC,
HMC8.VaR$bias.MC,
HMC8.VaR$bias.MCMC,
HMC10.VaR$bias.MC,
HMC10.VaR$bias.MCMC,
names=c("MC, d=4","HMC, d=4","MC, d=6","HMC, d=6","MC, d=8","HMC, d=8","MC, d=10","HMC, d=10"),
main = "(A1) VaR crisis event",
las=2,
at = c(1,2, 4,5, 7,8, 11,12),
col=c("blue","green","blue","green","blue","green","blue","green"),
ylab="Bias",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")

# Standard error
boxplot(
HMC4.VaR$MCse,
HMC4.VaR$MCMCse,
HMC6.VaR$MCse,
HMC6.VaR$MCMCse,
HMC8.VaR$MCse,
HMC8.VaR$MCMCse,
HMC10.VaR$MCse,
HMC10.VaR$MCMCse,
names=c("MC, d=4","HMC, d=4","MC, d=6","HMC, d=6","MC, d=8","HMC, d=8","MC, d=10","HMC, d=10"),
main="(A2) VaR crisis event",
las=2,
at = c(1,2, 4,5, 7,8, 11,12),
col=c("blue","green","blue","green","blue","green","blue","green"),
ylab="Standard error",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")

# MSE
boxplot(
HMC4.VaR$MSE.MC,
HMC4.VaR$MSE.MCMC,
HMC6.VaR$MSE.MC,
HMC6.VaR$MSE.MCMC,
HMC8.VaR$MSE.MC,
HMC8.VaR$MSE.MCMC,
HMC10.VaR$MSE.MC,
HMC10.VaR$MSE.MCMC,
names=c("MC, d=4","HMC, d=4","MC, d=6","HMC, d=6","MC, d=8","HMC, d=8","MC, d=10","HMC, d=10"),
ylim=c(0,0.05),
main = "(A3) VaR crisis event",
las=2,
at = c(1,2, 4,5, 7,8, 11,12),
col=c("blue","green","blue","green","blue","green","blue","green"),
ylab="Time adjusted MSE",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")

# Bias
boxplot(
HMC4.RVaR$bias.MC,
HMC4.RVaR$bias.MCMC,
GS4.RVaR$bias.MCMC,
HMC6.RVaR$bias.MC,
HMC6.RVaR$bias.MCMC,
GS6.RVaR$bias.MCMC,
HMC8.RVaR$bias.MC,
HMC8.RVaR$bias.MCMC,
GS8.RVaR$bias.MCMC,
HMC10.RVaR$bias.MC,
HMC10.RVaR$bias.MCMC,
GS10.RVaR$bias.MCMC,
names=c("MC, d=4","HMC, d=4","GS, d=4","MC, d=6","HMC, d=6","GS, d=6","MC, d=8","HMC, d=8","GS, d=8","MC, d=10","HMC, d=10","GS, d=10"),
main = "(A4) RVaR crisis event",
las=2,
at = c(1,2,3, 5,6,7, 9,10,11, 13,14,15),
col=c("blue","green","red","blue","green","red","blue","green","red","blue","green","red"),
ylab="Bias",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")

# Standard error
boxplot(
HMC4.RVaR$MCse,
HMC4.RVaR$MCMCse,
GS4.RVaR$MCMCse,
HMC6.RVaR$MCse,
HMC6.RVaR$MCMCse,
GS6.RVaR$MCMCse,
HMC8.RVaR$MCse,
HMC8.RVaR$MCMCse,
GS8.RVaR$MCMCse,
HMC10.RVaR$MCse,
HMC10.RVaR$MCMCse,
GS10.RVaR$MCMCse,
names=c("MC, d=4","HMC, d=4","GS, d=4","MC, d=6","HMC, d=6","GS, d=6","MC, d=8","HMC, d=8","GS, d=8","MC, d=10","HMC, d=10","GS, d=10"),
main = "(A5) RVaR crisis event",
las=2,
at = c(1,2,3, 5,6,7, 9,10,11, 13,14,15),
col=c("blue","green","red","blue","green","red","blue","green","red","blue","green","red"),
ylab="Standard error",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")

# MSE
boxplot(
HMC4.RVaR$MSE.MC,
HMC4.RVaR$MSE.MCMC,
GS4.RVaR$MSE.MCMC,
HMC6.RVaR$MSE.MC,
HMC6.RVaR$MSE.MCMC,
GS6.RVaR$MSE.MCMC,
HMC8.RVaR$MSE.MC,
HMC8.RVaR$MSE.MCMC,
GS8.RVaR$MSE.MCMC,
HMC10.RVaR$MSE.MC,
HMC10.RVaR$MSE.MCMC,
GS10.RVaR$MSE.MCMC,
names=c("MC, d=4","HMC, d=4","GS, d=4","MC, d=6","HMC, d=6","GS, d=6","MC, d=8","HMC, d=8","GS, d=8","MC, d=10","HMC, d=10","GS, d=10"),
main = "(A6) RVaR crisis event",
las=2,
at = c(1,2,3, 5,6,7, 9,10,11, 13,14,15),
col=c("blue","green","red","blue","green","red","blue","green","red","blue","green","red"),
ylab="Time adjusted MSE",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")

# Bias
boxplot(
HMC4.ES$bias.MC,
HMC4.ES$bias.MCMC,
GS4.ES$bias.MCMC,
HMC6.ES$bias.MC,
HMC6.ES$bias.MCMC,
GS6.ES$bias.MCMC,
HMC8.ES$bias.MC,
HMC8.ES$bias.MCMC,
GS8.ES$bias.MCMC,
HMC10.ES$bias.MC,
HMC10.ES$bias.MCMC,
GS10.ES$bias.MCMC,
names=c("MC, d=4","HMC, d=4","GS, d=4","MC, d=6","HMC, d=6","GS, d=6","MC, d=8","HMC, d=8","GS, d=8","MC, d=10","HMC, d=10","GS, d=10"),
main = "(A7) ES crisis event",
las=2,
at = c(1,2,3, 5,6,7, 9,10,11, 13,14,15),
col=c("blue","green","red","blue","green","red","blue","green","red","blue","green","red"),
ylab="Bias",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")



# Standard error
boxplot(
HMC4.ES$MCse,
HMC4.ES$MCMCse,
GS4.ES$MCMCse,
HMC6.ES$MCse,
HMC6.ES$MCMCse,
GS6.ES$MCMCse,
HMC8.ES$MCse,
HMC8.ES$MCMCse,
GS8.ES$MCMCse,
HMC10.ES$MCse,
HMC10.ES$MCMCse,
GS10.ES$MCMCse,
names=c("MC, d=4","HMC, d=4","GS, d=4","MC, d=6","HMC, d=6","GS, d=6","MC, d=8","HMC, d=8","GS, d=8","MC, d=10","HMC, d=10","GS, d=10"),
main = "(A8) ES crisis event",
las=2,
at = c(1,2,3, 5,6,7, 9,10,11, 13,14,15),
col=c("blue","green","red","blue","green","red","blue","green","red","blue","green","red"),
ylab="Standard error",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")

# MSE
boxplot(
HMC4.ES$MSE.MC,
HMC4.ES$MSE.MCMC,
GS4.ES$MSE.MCMC,
HMC6.ES$MSE.MC,
HMC6.ES$MSE.MCMC,
GS6.ES$MSE.MCMC,
HMC8.ES$MSE.MC,
HMC8.ES$MSE.MCMC,
GS8.ES$MSE.MCMC,
HMC10.ES$MSE.MC,
HMC10.ES$MSE.MCMC,
GS10.ES$MSE.MCMC,
names=c("MC, d=4","HMC, d=4","GS, d=4","MC, d=6","HMC, d=6","GS, d=6","MC, d=8","HMC, d=8","GS, d=8","MC, d=10","HMC, d=10","GS, d=10"),
main = "(A9) ES crisis event",
las=2,
at = c(1,2,3, 5,6,7, 9,10,11, 13,14,15),
col=c("blue","green","red","blue","green","red","blue","green","red","blue","green","red"),
ylab="Time adjusted MSE",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")



##############################################
## Comparison for different confidence levels
##############################################

# VaR
HMC.l1.VaR = CompareMCMC(d=5,levels = levelset.VaR[1],method.MCMC="HMC")
HMC.l2.VaR = CompareMCMC(d=5,levels = levelset.VaR[2],method.MCMC="HMC")
HMC.l3.VaR = CompareMCMC(d=5,levels = levelset.VaR[3],method.MCMC="HMC")
HMC.l4.VaR = CompareMCMC(d=5,levels = levelset.VaR[4],method.MCMC="HMC")


# Hyperparameters
HMC.l1.VaR$stepsize
HMC.l2.VaR$stepsize
HMC.l3.VaR$stepsize
HMC.l4.VaR$stepsize
HMC.l1.VaR$integration.time
HMC.l2.VaR$integration.time
HMC.l3.VaR$integration.time
HMC.l4.VaR$integration.time

# Run time
HMC.l1.VaR$runtime.MC
HMC.l1.VaR$runtime.MCMC
HMC.l2.VaR$runtime.MC
HMC.l2.VaR$runtime.MCMC
HMC.l3.VaR$runtime.MC
HMC.l3.VaR$runtime.MCMC
HMC.l4.VaR$runtime.MC
HMC.l4.VaR$runtime.MCMC
# stepsize, integration time and runtime of MC and MCMC do not vary 



# RVaR
HMC.l1.RVaR = CompareMCMC(d=5,levels = c(levelset.RVaR1[1],levelset.RVaR2[1]),method.MCMC="HMC")
HMC.l2.RVaR = CompareMCMC(d=5,levels = c(levelset.RVaR1[2],levelset.RVaR2[2]),method.MCMC="HMC")
HMC.l3.RVaR = CompareMCMC(d=5,levels = c(levelset.RVaR1[3],levelset.RVaR2[3]),method.MCMC="HMC")
HMC.l4.RVaR = CompareMCMC(d=5,levels = c(levelset.RVaR1[4],levelset.RVaR2[4]),method.MCMC="HMC")
GS.l1.RVaR = CompareMCMC(d=5,levels = c(levelset.RVaR1[1],levelset.RVaR2[1]),method.MCMC="GS")
GS.l2.RVaR = CompareMCMC(d=5,levels = c(levelset.RVaR1[2],levelset.RVaR2[2]),method.MCMC="GS")
GS.l3.RVaR = CompareMCMC(d=5,levels = c(levelset.RVaR1[3],levelset.RVaR2[3]),method.MCMC="GS")
GS.l4.RVaR = CompareMCMC(d=5,levels = c(levelset.RVaR1[4],levelset.RVaR2[4]),method.MCMC="GS")


#ES
HMC.l1.ES = CompareMCMC(d=5,levels = c(levelset.ES[1],1),method.MCMC="HMC")
HMC.l2.ES = CompareMCMC(d=5,levels = c(levelset.ES[2],1),method.MCMC="HMC")
HMC.l3.ES = CompareMCMC(d=5,levels = c(levelset.ES[3],1),method.MCMC="HMC")
HMC.l4.ES = CompareMCMC(d=5,levels = c(levelset.ES[4],1),method.MCMC="HMC")
GS.l1.ES = CompareMCMC(d=5,levels = c(levelset.ES[1],1),method.MCMC="GS")
GS.l2.ES = CompareMCMC(d=5,levels = c(levelset.ES[2],1),method.MCMC="GS")
GS.l3.ES = CompareMCMC(d=5,levels = c(levelset.ES[3],1),method.MCMC="GS")
GS.l4.ES = CompareMCMC(d=5,levels = c(levelset.ES[4],1),method.MCMC="GS")


# Boxplot

par(mfrow=c(3,3), mar = c(8, 5, 4, 2),mgp = c(3.5, 1, 0))

# Bias
boxplot(
HMC.l1.VaR$bias.MC,
HMC.l1.VaR$bias.MCMC,
HMC.l2.VaR$bias.MC,
HMC.l2.VaR$bias.MCMC,
HMC.l3.VaR$bias.MC,
HMC.l3.VaR$bias.MCMC,
HMC.l4.VaR$bias.MC,
HMC.l4.VaR$bias.MCMC,
names=c("MC, 0.9","HMC, 0.9","MC, 0.99","HMC, 0.99","MC, 0.999","HMC, 0.999","MC, 0.9999","HMC, 0.9999"),
main = "(B1) VaR crisis event",
las=2,
at = c(1,2, 4,5, 7,8, 11,12),
col=c("blue","green","blue","green","blue","green","blue","green"),
ylab="Bias",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")


# Standard error
boxplot(
HMC.l1.VaR$MCse,
HMC.l1.VaR$MCMCse,
HMC.l2.VaR$MCse,
HMC.l2.VaR$MCMCse,
HMC.l3.VaR$MCse,
HMC.l3.VaR$MCMCse,
HMC.l4.VaR$MCse,
HMC.l4.VaR$MCMCse,
names=c("MC, 0.9","HMC, 0.9","MC, 0.99","HMC, 0.99","MC, 0.999","HMC, 0.999","MC, 0.9999","HMC, 0.9999"),
main = "(B2) VaR crisis event",
las=2,
at = c(1,2, 4,5, 7,8, 11,12),
col=c("blue","green","blue","green","blue","green","blue","green"),
ylab="Standard error",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")


# MSE
boxplot(
HMC.l1.VaR$MSE.MC,
HMC.l1.VaR$MSE.MCMC,
HMC.l2.VaR$MSE.MC,
HMC.l2.VaR$MSE.MCMC,
HMC.l3.VaR$MSE.MC,
HMC.l3.VaR$MSE.MCMC,
HMC.l4.VaR$MSE.MC,
HMC.l4.VaR$MSE.MCMC,
names=c("MC, 0.9","HMC, 0.9","MC, 0.99","HMC, 0.99","MC, 0.999","HMC, 0.999","MC, 0.9999","HMC, 0.9999"),
main = "(B3) VaR crisis event",
las=2,
at = c(1,2, 4,5, 7,8, 11,12),
col=c("blue","green","blue","green","blue","green","blue","green"),
ylab="Time adjusted MSE",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")



# Bias
boxplot(
HMC.l1.RVaR$bias.MC,
HMC.l1.RVaR$bias.MCMC,
GS.l1.RVaR$bias.MCMC,
HMC.l2.RVaR$bias.MC,
HMC.l2.RVaR$bias.MCMC,
GS.l2.RVaR$bias.MCMC,
HMC.l3.RVaR$bias.MC,
HMC.l3.RVaR$bias.MCMC,
GS.l3.RVaR$bias.MCMC,
HMC.l4.RVaR$bias.MC,
HMC.l4.RVaR$bias.MCMC,
GS.l4.RVaR$bias.MCMC,
names=c("MC, 0.9, 0.9999)","HMC, (0.9, 0.9999)","GS, (0.9, 0.9999)",
  "MC, (0.9, 0.99)","HMC, (0.9, 0.99)","GS, (0.9, 0.99)",
  "MC, (0.99, 0.999)","HMC, (0.99, 0.999)","GS, (0.99, 0.999)",
  "MC, (0.999, 0.9999)","HMC, (0.999, 0.9999)","GS, (0.999, 0.9999)"),
main = "(B4) RVaR crisis event",
las=2,
at = c(1,2,3, 5,6,7, 9,10,11, 13,14,15),
col=c("blue","green","red","blue","green","red","blue","green","red","blue","green","red"),
ylab="Bias",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")




# Standard error
boxplot(
HMC.l1.RVaR$MCse,
HMC.l1.RVaR$MCMCse,
GS.l1.RVaR$MCMCse,
HMC.l2.RVaR$MCse,
HMC.l2.RVaR$MCMCse,
GS.l2.RVaR$MCMCse,
HMC.l3.RVaR$MCse,
HMC.l3.RVaR$MCMCse,
GS.l3.RVaR$MCMCse,
HMC.l4.RVaR$MCse,
HMC.l4.RVaR$MCMCse,
GS.l4.RVaR$MCMCse,
names=c("MC, 0.9, 0.9999)","HMC, (0.9, 0.9999)","GS, (0.9, 0.9999)",
  "MC, (0.9, 0.99)","HMC, (0.9, 0.99)","GS, (0.9, 0.99)",
  "MC, (0.99, 0.999)","HMC, (0.99, 0.999)","GS, (0.99, 0.999)",
  "MC, (0.999, 0.9999)","HMC, (0.999, 0.9999)","GS, (0.999, 0.9999)"),
main = "(B5) RVaR crisis event",
las=2,
at = c(1,2,3, 5,6,7, 9,10,11, 13,14,15),
col=c("blue","green","red","blue","green","red","blue","green","red","blue","green","red"),
ylab="Standard error",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")


# MSE
boxplot(
HMC.l1.RVaR$MSE.MC,
HMC.l1.RVaR$MSE.MCMC,
GS.l1.RVaR$MSE.MCMC,
HMC.l2.RVaR$MSE.MC,
HMC.l2.RVaR$MSE.MCMC,
GS.l2.RVaR$MSE.MCMC,
HMC.l3.RVaR$MSE.MC,
HMC.l3.RVaR$MSE.MCMC,
GS.l3.RVaR$MSE.MCMC,
HMC.l4.RVaR$MSE.MC,
HMC.l4.RVaR$MSE.MCMC,
GS.l4.RVaR$MSE.MCMC,
names=c("MC, 0.9, 0.9999)","HMC, (0.9, 0.9999)","GS, (0.9, 0.9999)",
  "MC, (0.9, 0.99)","HMC, (0.9, 0.99)","GS, (0.9, 0.99)",
  "MC, (0.99, 0.999)","HMC, (0.99, 0.999)","GS, (0.99, 0.999)",
  "MC, (0.999, 0.9999)","HMC, (0.999, 0.9999)","GS, (0.999, 0.9999)"),
main = "(B6) RVaR crisis event",
las=2,
at = c(1,2,3, 5,6,7, 9,10,11, 13,14,15),
col=c("blue","green","red","blue","green","red","blue","green","red","blue","green","red"),
ylab="Time adjusted MSE",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")




# Bias
boxplot(
HMC.l1.ES$bias.MC,
HMC.l1.ES$bias.MCMC,
GS.l1.ES$bias.MCMC,
HMC.l2.ES$bias.MC,
HMC.l2.ES$bias.MCMC,
GS.l2.ES$bias.MCMC,
HMC.l3.ES$bias.MC,
HMC.l3.ES$bias.MCMC,
GS.l3.ES$bias.MCMC,
HMC.l4.ES$bias.MC,
HMC.l4.ES$bias.MCMC,
GS.l4.ES$bias.MCMC,
names=c("MC, 0.9","HMC, 0.9","GS, 0.9","MC, 0.99","HMC, 0.99","GS, 0.99","MC, 0.999","HMC, 0.999","GS, 0.999","MC, 0.9999","HMC, 0.9999","GS, 0.9999"),
main = "(B7) ES crisis event",
las=2,
at = c(1,2,3, 5,6,7, 9,10,11, 13,14,15),
col=c("blue","green","red","blue","green","red","blue","green","red","blue","green","red"),
ylab="Bias",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")

# Standard error
boxplot(
HMC.l1.ES$MCse,
HMC.l1.ES$MCMCse,
GS.l1.ES$MCMCse,
HMC.l2.ES$MCse,
HMC.l2.ES$MCMCse,
GS.l2.ES$MCMCse,
HMC.l3.ES$MCse,
HMC.l3.ES$MCMCse,
GS.l3.ES$MCMCse,
HMC.l4.ES$MCse,
HMC.l4.ES$MCMCse,
GS.l4.ES$MCMCse,
names=c("MC, 0.9","HMC, 0.9","GS, 0.9","MC, 0.99","HMC, 0.99","GS, 0.99","MC, 0.999","HMC, 0.999","GS, 0.999","MC, 0.9999","HMC, 0.9999","GS, 0.9999"),
main = "(B8) ES crisis event",
las=2,
at = c(1,2,3, 5,6,7, 9,10,11, 13,14,15),
col=c("blue","green","red","blue","green","red","blue","green","red","blue","green","red"),
ylab="Standard error",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")

# MSE
boxplot(
HMC.l1.ES$MSE.MC,
HMC.l1.ES$MSE.MCMC,
GS.l1.ES$MSE.MCMC,
HMC.l2.ES$MSE.MC,
HMC.l2.ES$MSE.MCMC,
GS.l2.ES$MSE.MCMC,
HMC.l3.ES$MSE.MC,
HMC.l3.ES$MSE.MCMC,
GS.l3.ES$MSE.MCMC,
HMC.l4.ES$MSE.MC,
HMC.l4.ES$MSE.MCMC,
GS.l4.ES$MSE.MCMC,
names=c("MC, 0.9","HMC, 0.9","GS, 0.9","MC, 0.99","HMC, 0.99","GS, 0.99","MC, 0.999","HMC, 0.999","GS, 0.999","MC, 0.9999","HMC, 0.9999","GS, 0.9999"),
main = "(B9) ES crisis event",
las=2,
at = c(1,2,3, 5,6,7, 9,10,11, 13,14,15),
col=c("blue","green","red","blue","green","red","blue","green","red","blue","green","red"),
ylab="Time adjusted MSE",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")


#################################################
## Comparison for different epsilon and T
#################################################

# multiplication/division coefficients
c.eps <- 10
c.T <- 2

# VaR
HMC.VaR.opt = HMC.l1.VaR
# HMC.l1.VaR = CompareMCMC(d=5,levels = levelset.VaR[1],method.MCMC="HMC")
eps.VaR.opt = HMC.VaR.opt$stepsize
T.VaR.opt = HMC.VaR.opt$integration.time
HMC.VaR.mm = CompareMCMC(d=5,levels = levelset.VaR[1],method.MCMC="HMC",eps.HMC=eps.VaR.opt*c.eps,T.HMC=T.VaR.opt*c.T)
HMC.VaR.md = CompareMCMC(d=5,levels = levelset.VaR[1],method.MCMC="HMC",eps.HMC=eps.VaR.opt*c.eps,T.HMC=floor(T.VaR.opt/c.T))
HMC.VaR.dm = CompareMCMC(d=5,levels = levelset.VaR[1],method.MCMC="HMC",eps.HMC=eps.VaR.opt/c.eps,T.HMC=T.VaR.opt*c.T)
HMC.VaR.dd = CompareMCMC(d=5,levels = levelset.VaR[1],method.MCMC="HMC",eps.HMC=eps.VaR.opt/c.eps,T.HMC=floor(T.VaR.opt/c.T))

# RVaR
HMC.RVaR.opt = HMC.l2.RVaR
# HMC.l1.RVaR = CompareMCMC(d=5,levels = c(levelset.RVaR1[1],levelset.RVaR2[1]),method.MCMC="HMC")
eps.RVaR.opt = HMC.RVaR.opt$stepsize
T.RVaR.opt = HMC.RVaR.opt$integration.time
HMC.RVaR.mm = CompareMCMC(d=5,levels = c(levelset.RVaR1[2],levelset.RVaR2[2]),method.MCMC="HMC",eps.HMC=eps.RVaR.opt*c.eps,T.HMC=T.RVaR.opt*c.T)
HMC.RVaR.md = CompareMCMC(d=5,levels = c(levelset.RVaR1[2],levelset.RVaR2[2]),method.MCMC="HMC",eps.HMC=eps.RVaR.opt*c.eps,T.HMC=floor(T.RVaR.opt/c.T))
HMC.RVaR.dm = CompareMCMC(d=5,levels = c(levelset.RVaR1[2],levelset.RVaR2[2]),method.MCMC="HMC",eps.HMC=eps.RVaR.opt/c.eps,T.HMC=T.RVaR.opt*c.T)
HMC.RVaR.dd = CompareMCMC(d=5,levels = c(levelset.RVaR1[2],levelset.RVaR2[2]),method.MCMC="HMC",eps.HMC=eps.RVaR.opt/c.eps,T.HMC=floor(T.RVaR.opt/c.T))

#ES
HMC.ES.opt = HMC.l1.ES
# HMC.l1.ES = CompareMCMC(d=5,levels = c(levelset.ES[1],1),method.MCMC="HMC")
eps.ES.opt = HMC.ES.opt$stepsize
T.ES.opt = HMC.ES.opt$integration.time
HMC.ES.mm = CompareMCMC(d=5,levels = c(levelset.ES[1],1),method.MCMC="HMC",eps.HMC=eps.ES.opt*c.eps,T.HMC=T.ES.opt*c.T)
HMC.ES.md = CompareMCMC(d=5,levels = c(levelset.ES[1],1),method.MCMC="HMC",eps.HMC=eps.ES.opt*c.eps,T.HMC=floor(T.ES.opt/c.T))
HMC.ES.dm = CompareMCMC(d=5,levels = c(levelset.ES[1],1),method.MCMC="HMC",eps.HMC=eps.ES.opt/c.eps,T.HMC=T.ES.opt*c.T)
HMC.ES.dd = CompareMCMC(d=5,levels = c(levelset.ES[1],1),method.MCMC="HMC",eps.HMC=eps.ES.opt/c.eps,T.HMC=floor(T.ES.opt/c.T))

#  oxplot

par(mfrow=c(3,3), mar = c(6, 5, 4, 2),mgp = c(3.5, 1, 0))

# Bias
boxplot(
HMC.VaR.opt$bias.MC,
HMC.VaR.opt$bias.MCMC,
HMC.VaR.mm$bias.MCMC,
HMC.VaR.md$bias.MCMC,
HMC.VaR.dm$bias.MCMC,
HMC.VaR.dd$bias.MCMC,
names=c("MC","HMC, opt","HMC, mm","HMC, md","HMC, dm","HMC, dd"),
main = "(C1) VaR crisis event",
las=2,
at = c(1,2,3,4,5,6),
col=c("blue","green","green","green","green","green"),
ylab="Bias",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")


# Standard error
boxplot(
HMC.VaR.opt$MCse,
HMC.VaR.opt$MCMCse,
HMC.VaR.mm$MCMCse,
HMC.VaR.md$MCMCse,
HMC.VaR.dm$MCMCse,
HMC.VaR.dd$MCMCse,
names=c("MC","HMC, opt","HMC, mm","HMC, md","HMC, dm","HMC, dd"),
main = "(C2) VaR crisis event",
las=2,
at = c(1,2,3,4,5,6),
col=c("blue","green","green","green","green","green"),
ylab="Standard error",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")

# MSE
boxplot(
HMC.VaR.opt$MSE.MC,
HMC.VaR.opt$MSE.MCMC,
HMC.VaR.mm$MSE.MCMC,
HMC.VaR.md$MSE.MCMC,
HMC.VaR.dm$MSE.MCMC,
HMC.VaR.dd$MSE.MCMC,
names=c("MC","HMC, opt","HMC, mm","HMC, md","HMC, dm","HMC, dd"),
main = "(C3) VaR crisis event",
las=2,
at = c(1,2,3,4,5,6),
col=c("blue","green","green","green","green","green"),
ylab="Time adjusted MSE",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")





# Bias
boxplot(
HMC.RVaR.opt$bias.MC,
HMC.RVaR.opt$bias.MCMC,
HMC.RVaR.mm$bias.MCMC,
HMC.RVaR.md$bias.MCMC,
HMC.RVaR.dm$bias.MCMC,
HMC.RVaR.dd$bias.MCMC,
names=c("MC","HMC, opt","HMC, mm","HMC, md","HMC, dm","HMC, dd"),
main = "(C4) RVaR crisis event",
las=2,
at = c(1,2,3,4,5,6),
col=c("blue","green","green","green","green","green"),
ylab="Bias",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")

# Standard error
boxplot(
HMC.RVaR.opt$MCse,
HMC.RVaR.opt$MCMCse,
HMC.RVaR.mm$MCMCse,
HMC.RVaR.md$MCMCse,
HMC.RVaR.dm$MCMCse,
HMC.RVaR.dd$MCMCse,
names=c("MC","HMC, opt","HMC, mm","HMC, md","HMC, dm","HMC, dd"),
main = "(C5) RVaR crisis event",
las=2,
at = c(1,2,3,4,5,6),
col=c("blue","green","green","green","green","green"),
ylab="Standard error",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")

# MSE
boxplot(
HMC.RVaR.opt$MSE.MC,
HMC.RVaR.opt$MSE.MCMC,
HMC.RVaR.mm$MSE.MCMC,
HMC.RVaR.md$MSE.MCMC,
HMC.RVaR.dm$MSE.MCMC,
HMC.RVaR.dd$MSE.MCMC,
names=c("MC","HMC, opt","HMC, mm","HMC, md","HMC, dm","HMC, dd"),
main = "(C6) RVaR crisis event",
las=2,
at = c(1,2,3,4,5,6),
col=c("blue","green","green","green","green","green"),
ylab="Time adjusted MSE",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")

# Bias
boxplot(
HMC.ES.opt$bias.MC,
HMC.ES.opt$bias.MCMC,
HMC.ES.mm$bias.MCMC,
HMC.ES.md$bias.MCMC,
HMC.ES.dm$bias.MCMC,
HMC.ES.dd$bias.MCMC,
names=c("MC","HMC, opt","HMC, mm","HMC, md","HMC, dm","HMC, dd"),
main = "(C7) ES crisis event",
las=2,
at = c(1,2,3,4,5,6),
col=c("blue","green","green","green","green","green"),
ylab="Bias",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")

# Standard error
boxplot(
HMC.ES.opt$MCse,
HMC.ES.opt$MCMCse,
HMC.ES.mm$MCMCse,
HMC.ES.md$MCMCse,
HMC.ES.dm$MCMCse,
HMC.ES.dd$MCMCse,
names=c("MC","HMC, opt","HMC, mm","HMC, md","HMC, dm","HMC, dd"),
main = "(C8) ES crisis event",
las=2,
at = c(1,2,3,4,5,6),
col=c("blue","green","green","green","green","green"),
ylab="Standard error",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")

# MSE
boxplot(
HMC.ES.opt$MSE.MC,
HMC.ES.opt$MSE.MCMC,
HMC.ES.mm$MSE.MCMC,
HMC.ES.md$MSE.MCMC,
HMC.ES.dm$MSE.MCMC,
HMC.ES.dd$MSE.MCMC,
names=c("MC","HMC, opt","HMC, mm","HMC, md","HMC, dm","HMC, dd"),
main = "(C9) ES crisis event",
las=2,
at = c(1,2,3,4,5,6),
col=c("blue","green","green","green","green","green"),
ylab="Time adjusted MSE",
cex.lab=1.5
)
abline(h =0,col="gray",lty="dotted")


######################
## Save data
#####################

# save
LevelMat <- rbind(
HMC.l1.VaR$bias.MC,
HMC.l1.VaR$bias.MCMC,
HMC.l2.VaR$bias.MC,
HMC.l2.VaR$bias.MCMC,
HMC.l3.VaR$bias.MC,
HMC.l3.VaR$bias.MCMC,
HMC.l4.VaR$bias.MC,
HMC.l4.VaR$bias.MCMC,
#
HMC.l1.VaR$MCse,
HMC.l1.VaR$MCMCse,
HMC.l2.VaR$MCse,
HMC.l2.VaR$MCMCse,
HMC.l3.VaR$MCse,
HMC.l3.VaR$MCMCse,
HMC.l4.VaR$MCse,
HMC.l4.VaR$MCMCse,
#
HMC.l1.VaR$MSE.MC,
HMC.l1.VaR$MSE.MCMC,
HMC.l2.VaR$MSE.MC,
HMC.l2.VaR$MSE.MCMC,
HMC.l3.VaR$MSE.MC,
HMC.l3.VaR$MSE.MCMC,
HMC.l4.VaR$MSE.MC,
HMC.l4.VaR$MSE.MCMC,
#
HMC.l1.RVaR$bias.MC,
HMC.l1.RVaR$bias.MCMC,
GS.l1.RVaR$bias.MCMC,
HMC.l2.RVaR$bias.MC,
HMC.l2.RVaR$bias.MCMC,
GS.l2.RVaR$bias.MCMC,
HMC.l3.RVaR$bias.MC,
HMC.l3.RVaR$bias.MCMC,
GS.l3.RVaR$bias.MCMC,
HMC.l4.RVaR$bias.MC,
HMC.l4.RVaR$bias.MCMC,
GS.l4.RVaR$bias.MCMC,
#
HMC.l1.RVaR$MCse,
HMC.l1.RVaR$MCMCse,
GS.l1.RVaR$MCMCse,
HMC.l2.RVaR$MCse,
HMC.l2.RVaR$MCMCse,
GS.l2.RVaR$MCMCse,
HMC.l3.RVaR$MCse,
HMC.l3.RVaR$MCMCse,
GS.l3.RVaR$MCMCse,
HMC.l4.RVaR$MCse,
HMC.l4.RVaR$MCMCse,
GS.l4.RVaR$MCMCse,
#
HMC.l1.RVaR$MSE.MC,
HMC.l1.RVaR$MSE.MCMC,
GS.l1.RVaR$MSE.MCMC,
HMC.l2.RVaR$MSE.MC,
HMC.l2.RVaR$MSE.MCMC,
GS.l2.RVaR$MSE.MCMC,
HMC.l3.RVaR$MSE.MC,
HMC.l3.RVaR$MSE.MCMC,
GS.l3.RVaR$MSE.MCMC,
HMC.l4.RVaR$MSE.MC,
HMC.l4.RVaR$MSE.MCMC,
GS.l4.RVaR$MSE.MCMC,
#
HMC.l1.ES$bias.MC,
HMC.l1.ES$bias.MCMC,
GS.l1.ES$bias.MCMC,
HMC.l2.ES$bias.MC,
HMC.l2.ES$bias.MCMC,
GS.l2.ES$bias.MCMC,
HMC.l3.ES$bias.MC,
HMC.l3.ES$bias.MCMC,
GS.l3.ES$bias.MCMC,
HMC.l4.ES$bias.MC,
HMC.l4.ES$bias.MCMC,
GS.l4.ES$bias.MCMC,
#
HMC.l1.ES$MCse,
HMC.l1.ES$MCMCse,
GS.l1.ES$MCMCse,
HMC.l2.ES$MCse,
HMC.l2.ES$MCMCse,
GS.l2.ES$MCMCse,
HMC.l3.ES$MCse,
HMC.l3.ES$MCMCse,
GS.l3.ES$MCMCse,
HMC.l4.ES$MCse,
HMC.l4.ES$MCMCse,
GS.l4.ES$MCMCse,
#
HMC.l1.ES$MSE.MC,
HMC.l1.ES$MSE.MCMC,
GS.l1.ES$MSE.MCMC,
HMC.l2.ES$MSE.MC,
HMC.l2.ES$MSE.MCMC,
GS.l2.ES$MSE.MCMC,
HMC.l3.ES$MSE.MC,
HMC.l3.ES$MSE.MCMC,
GS.l3.ES$MSE.MCMC,
HMC.l4.ES$MSE.MC,
HMC.l4.ES$MSE.MCMC,
GS.l4.ES$MSE.MCMC
)

write.csv(LevelMat, "LevelMat.csv", quote=FALSE, row.names=FALSE)
#read.csv("LevelMat.csv",header=TRUE, skip=0)


#
DimMat <- matrix(0, ncol = 10, nrow= 96)


DimMat[1,1:4] <- HMC4.VaR$bias.MC
DimMat[2,1:4] <- HMC4.VaR$bias.MCMC
DimMat[3,1:6] <- HMC6.VaR$bias.MC
DimMat[4,1:6] <- HMC6.VaR$bias.MCMC
DimMat[5,1:8] <- HMC8.VaR$bias.MC
DimMat[6,1:8] <- HMC8.VaR$bias.MCMC
DimMat[7,1:10] <- HMC10.VaR$bias.MC
DimMat[8,1:10] <- HMC10.VaR$bias.MCMC
#
DimMat[9,1:4] <- HMC4.VaR$MCse
DimMat[10,1:4] <- HMC4.VaR$MCMCse
DimMat[11,1:6] <- HMC6.VaR$MCse
DimMat[12,1:6] <- HMC6.VaR$MCMCse
DimMat[13,1:8] <- HMC8.VaR$MCse
DimMat[14,1:8] <- HMC8.VaR$MCMCse
DimMat[15,1:10] <- HMC10.VaR$MCse
DimMat[16,1:10] <- HMC10.VaR$MCMCse
#
DimMat[17,1:4] <- HMC4.VaR$MSE.MC
DimMat[18,1:4] <- HMC4.VaR$MSE.MCMC
DimMat[19,1:6] <- HMC6.VaR$MSE.MC
DimMat[20,1:6] <- HMC6.VaR$MSE.MCMC
DimMat[21,1:8] <- HMC8.VaR$MSE.MC
DimMat[22,1:8] <- HMC8.VaR$MSE.MCMC
DimMat[23,1:10] <- HMC10.VaR$MSE.MC
DimMat[24,1:10] <- HMC10.VaR$MSE.MCMC
#
DimMat[25,1:4] <- HMC4.RVaR$bias.MC
DimMat[26,1:4] <- HMC4.RVaR$bias.MCMC
DimMat[27,1:4] <- GS4.RVaR$bias.MCMC
DimMat[28,1:6] <- HMC6.RVaR$bias.MC
DimMat[29,1:6] <- HMC6.RVaR$bias.MCMC
DimMat[30,1:6] <- GS6.RVaR$bias.MCMC
DimMat[31,1:8] <- HMC8.RVaR$bias.MC
DimMat[32,1:8] <- HMC8.RVaR$bias.MCMC
DimMat[33,1:8] <- GS8.RVaR$bias.MCMC
DimMat[34,1:10] <- HMC10.RVaR$bias.MC
DimMat[35,1:10] <- HMC10.RVaR$bias.MCMC
DimMat[36,1:10] <- GS10.RVaR$bias.MCMC
#
DimMat[37,1:4] <- HMC4.RVaR$MCse
DimMat[38,1:4] <- HMC4.RVaR$MCMCse
DimMat[39,1:4] <- GS4.RVaR$MCMCse
DimMat[40,1:6] <- HMC6.RVaR$MCse
DimMat[41,1:6] <- HMC6.RVaR$MCMCse
DimMat[42,1:6] <- GS6.RVaR$MCMCse
DimMat[43,1:8] <- HMC8.RVaR$MCse
DimMat[44,1:8] <- HMC8.RVaR$MCMCse
DimMat[45,1:8] <- GS8.RVaR$MCMCse
DimMat[46,1:10] <- HMC10.RVaR$MCse
DimMat[47,1:10] <- HMC10.RVaR$MCMCse
DimMat[48,1:10] <- GS10.RVaR$MCMCse
#
DimMat[49,1:4] <- HMC4.RVaR$MSE.MC
DimMat[50,1:4] <- HMC4.RVaR$MSE.MCMC
DimMat[51,1:4] <- GS4.RVaR$MSE.MCMC
DimMat[52,1:6] <- HMC6.RVaR$MSE.MC
DimMat[53,1:6] <- HMC6.RVaR$MSE.MCMC
DimMat[54,1:6] <- GS6.RVaR$MSE.MCMC
DimMat[55,1:8] <- HMC8.RVaR$MSE.MC
DimMat[56,1:8] <- HMC8.RVaR$MSE.MCMC
DimMat[57,1:8] <- GS8.RVaR$MSE.MCMC
DimMat[58,1:10] <- HMC10.RVaR$MSE.MC
DimMat[59,1:10] <- HMC10.RVaR$MSE.MCMC
DimMat[60,1:10] <- GS10.RVaR$MSE.MCMC
#
DimMat[61,1:4] <- HMC4.ES$bias.MC
DimMat[62,1:4] <- HMC4.ES$bias.MCMC
DimMat[63,1:4] <- GS4.ES$bias.MCMC
DimMat[64,1:6] <- HMC6.ES$bias.MC
DimMat[65,1:6] <- HMC6.ES$bias.MCMC
DimMat[66,1:6] <- GS6.ES$bias.MCMC
DimMat[67,1:8] <- HMC8.ES$bias.MC
DimMat[68,1:8] <- HMC8.ES$bias.MCMC
DimMat[69,1:8] <- GS8.ES$bias.MCMC
DimMat[70,1:10] <- HMC10.ES$bias.MC
DimMat[71,1:10] <- HMC10.ES$bias.MCMC
DimMat[72,1:10] <- GS10.ES$bias.MCMC
#
DimMat[73,1:4] <- HMC4.ES$MCse
DimMat[74,1:4] <- HMC4.ES$MCMCse
DimMat[75,1:4] <- GS4.ES$MCMCse
DimMat[76,1:6] <- HMC6.ES$MCse
DimMat[77,1:6] <- HMC6.ES$MCMCse
DimMat[78,1:6] <- GS6.ES$MCMCse
DimMat[79,1:8] <- HMC8.ES$MCse
DimMat[80,1:8] <- HMC8.ES$MCMCse
DimMat[81,1:8] <- GS8.ES$MCMCse
DimMat[82,1:10] <- HMC10.ES$MCse
DimMat[83,1:10] <- HMC10.ES$MCMCse
DimMat[84,1:10] <- GS10.ES$MCMCse
#
DimMat[85,1:4] <- HMC4.ES$MSE.MC
DimMat[86,1:4] <- HMC4.ES$MSE.MCMC
DimMat[87,1:4] <- GS4.ES$MSE.MCMC
DimMat[88,1:6] <- HMC6.ES$MSE.MC
DimMat[89,1:6] <- HMC6.ES$MSE.MCMC
DimMat[90,1:6] <- GS6.ES$MSE.MCMC
DimMat[91,1:8] <- HMC8.ES$MSE.MC
DimMat[92,1:8] <- HMC8.ES$MSE.MCMC
DimMat[93,1:8] <- GS8.ES$MSE.MCMC
DimMat[94,1:10] <- HMC10.ES$MSE.MC
DimMat[95,1:10] <- HMC10.ES$MSE.MCMC
DimMat[96,1:10] <- GS10.ES$MSE.MCMC

write.csv(DimMat, "DimMat.csv", quote=FALSE, row.names=FALSE)
#read.csv("DimMat.csv",header=TRUE, skip=0)


#

OptMat <- rbind(
HMC.VaR.opt$bias.MC,
HMC.VaR.opt$bias.MCMC,
HMC.VaR.mm$bias.MCMC,
HMC.VaR.md$bias.MCMC,
HMC.VaR.dm$bias.MCMC,
HMC.VaR.dd$bias.MCMC,
#
HMC.VaR.opt$MCse,
HMC.VaR.opt$MCMCse,
HMC.VaR.mm$MCMCse,
HMC.VaR.md$MCMCse,
HMC.VaR.dm$MCMCse,
HMC.VaR.dd$MCMCse,
#
HMC.VaR.opt$MSE.MC,
HMC.VaR.opt$MSE.MCMC,
HMC.VaR.mm$MSE.MCMC,
HMC.VaR.md$MSE.MCMC,
HMC.VaR.dm$MSE.MCMC,
HMC.VaR.dd$MSE.MCMC,
#
HMC.RVaR.opt$bias.MC,
HMC.RVaR.opt$bias.MCMC,
HMC.RVaR.mm$bias.MCMC,
HMC.RVaR.md$bias.MCMC,
HMC.RVaR.dm$bias.MCMC,
HMC.RVaR.dd$bias.MCMC,
#
HMC.RVaR.opt$MCse,
HMC.RVaR.opt$MCMCse,
HMC.RVaR.mm$MCMCse,
HMC.RVaR.md$MCMCse,
HMC.RVaR.dm$MCMCse,
HMC.RVaR.dd$MCMCse,
#
HMC.RVaR.opt$MSE.MC,
HMC.RVaR.opt$MSE.MCMC,
HMC.RVaR.mm$MSE.MCMC,
HMC.RVaR.md$MSE.MCMC,
HMC.RVaR.dm$MSE.MCMC,
HMC.RVaR.dd$MSE.MCMC,
#
HMC.ES.opt$bias.MC,
HMC.ES.opt$bias.MCMC,
HMC.ES.mm$bias.MCMC,
HMC.ES.md$bias.MCMC,
HMC.ES.dm$bias.MCMC,
HMC.ES.dd$bias.MCMC,
# 
HMC.ES.opt$MCse,
HMC.ES.opt$MCMCse,
HMC.ES.mm$MCMCse,
HMC.ES.md$MCMCse,
HMC.ES.dm$MCMCse,
HMC.ES.dd$MCMCse,
# 
HMC.ES.opt$MSE.MC,
HMC.ES.opt$MSE.MCMC,
HMC.ES.mm$MSE.MCMC,
HMC.ES.md$MSE.MCMC,
HMC.ES.dm$MSE.MCMC,
HMC.ES.dd$MSE.MCMC
)

write.csv(OptMat, "OptMat.csv", quote=FALSE, row.names=FALSE)
#read.csv("OptMat.csv",header=TRUE, skip=0)


## Reproducing boxplots for paper 

### Setup ######################################################################

## Packages
doPDF <- require(crop)

## Data
LevelMat <- as.matrix(read.csv("LevelMat.csv", header = TRUE, skip = 0))
DimMat   <- as.matrix(read.csv("DimMat.csv",   header = TRUE, skip = 0))
OptMat   <- as.matrix(read.csv("OptMat.csv",   header = TRUE, skip = 0))


### Auxiliary functions ########################################################

## Boxplot for basic data
boxplot_basic <- function(..., names, col, ylab, text)
{
    data <- list(...)
    len <- length(data)
    stopifnot(length(names) == len)
    boxplot(data, names = names, main = "", las = 3, at = seq_len(len), col = col, ylab = ylab)
    mtext(text, side = 4, line = 0.5, adj = 0, cex = 0.7)
}

## Boxplot for grouped data
boxplot_groups <- function(..., names, col, ylab, text)
{
    data <- list(...)
    len <- length(data)
    stopifnot(length(names) == len)
    at <- if(len == 8) {
              c(1,2, 4,5, 7,8, 11,12)
          } else if(len == 12) {
              c(1,2,3, 5,6,7, 9,10,11, 13,14,15)
          } else stop("Wrong length of '...'")
    boxplot(data, names = names, main = "", las = 3, at = at, col = col, ylab = ylab)
    mtext(text, side = 4, line = 0.5, adj = 0, cex = 0.7)
}

## Create x-axis labels
xlabels <- function(methods, info)
    as.vector(outer(methods, info, FUN = function(x, y) paste(x, y, sep = ", ")))


### Figure 4: Comparison in d ##################################################

if(doPDF) pdf(file = (file <- "DimPlot.pdf"), width = 10, height = 12)
## Layout
layout(matrix(1:9, ncol = 3, byrow = TRUE))
opar <- par(pty = "s")
## Plot
## (1,1) Bias
boxplot_groups(DimMat[1, 1:4],  DimMat[2, 1:4],
               DimMat[3, 1:6],  DimMat[4, 1:6],
               DimMat[5, 1:8],  DimMat[6, 1:8],
               DimMat[7, 1:10], DimMat[8, 1:10],
               names = xlabels(c("MC", "HMC"),
                               info = c("d=4", "d=6", "d=8", "d=10")),
               col = rep(c("blue", "green"), 4),
               ylab = "Bias", text = "(A1) VaR crisis event")
## (1,2) SE
boxplot_groups(DimMat[9,  1:4],  DimMat[10, 1:4],
               DimMat[11, 1:6],  DimMat[12, 1:6],
               DimMat[13, 1:8],  DimMat[14, 1:8],
               DimMat[15, 1:10], DimMat[16, 1:10],
               names = xlabels(c("MC", "HMC"),
                               info = c("d=4", "d=6", "d=8", "d=10")),
               col = rep(c("blue", "green"), 4),
               ylab = "Standard error", text = "(A2) VaR crisis event")
## (1,3) Time-adjusted MSE
boxplot_groups(DimMat[17, 1:4],  DimMat[18, 1:4],
               DimMat[19, 1:6],  DimMat[20, 1:6],
               DimMat[21, 1:8],  DimMat[22, 1:8],
               DimMat[23, 1:10], DimMat[24, 1:10],
               names = xlabels(c("MC", "HMC"),
                               info = c("d=4", "d=6", "d=8", "d=10")),
               col = rep(c("blue", "green"), 4),
               ylab = "Time-adjusted MSE", text = "(A3) VaR crisis event")
## (2,1) Bias
boxplot_groups(DimMat[25, 1:4],  DimMat[26, 1:4],  DimMat[27, 1:4],
               DimMat[28, 1:6],  DimMat[29, 1:6],  DimMat[30, 1:6],
               DimMat[31, 1:8],  DimMat[32, 1:8],  DimMat[33, 1:8],
               DimMat[34, 1:10], DimMat[35, 1:10], DimMat[36, 1:10],
               names = xlabels(c("MC", "HMC", "GS"),
                               info = c("d=4", "d=6", "d=8", "d=10")),
               col = rep(c("blue", "green", "red"), 4),
               ylab = "Bias", text = "(A4) RVaR crisis event")
## (2,2) SE
boxplot_groups(DimMat[37, 1:4],  DimMat[38, 1:4],  DimMat[39, 1:4],
               DimMat[40, 1:6],  DimMat[41, 1:6],  DimMat[42, 1:6],
               DimMat[43, 1:8],  DimMat[44, 1:8],  DimMat[45, 1:8],
               DimMat[46, 1:10], DimMat[47, 1:10], DimMat[48, 1:10],
               names = xlabels(c("MC", "HMC", "GS"),
                               info = c("d=4", "d=6", "d=8", "d=10")),
               col = rep(c("blue", "green", "red"), 4),
               ylab = "Standard error", text = "(A5) RVaR crisis event")
## (2,3) Time-adjusted MSE
boxplot_groups(DimMat[49, 1:4],  DimMat[50, 1:4],  DimMat[51, 1:4],
               DimMat[52, 1:6],  DimMat[53, 1:6],  DimMat[54, 1:6],
               DimMat[55, 1:8],  DimMat[56, 1:8],  DimMat[57, 1:8],
               DimMat[58, 1:10], DimMat[59, 1:10], DimMat[60, 1:10],
               names = xlabels(c("MC", "HMC", "GS"),
                               info = c("d=4", "d=6", "d=8", "d=10")),
               col = rep(c("blue", "green", "red"), 4),
               ylab = "Time-adjusted MSE", text = "(A6) RVaR crisis event")
## (3,1) Bias
boxplot_groups(DimMat[61, 1:4],  DimMat[62, 1:4],  DimMat[63, 1:4],
               DimMat[64, 1:6],  DimMat[65, 1:6],  DimMat[66, 1:6],
               DimMat[67, 1:8],  DimMat[68, 1:8],  DimMat[69, 1:8],
               DimMat[70, 1:10], DimMat[71, 1:10], DimMat[72, 1:10],
               names = xlabels(c("MC", "HMC", "GS"),
                               info = c("d=4", "d=6", "d=8", "d=10")),
               col = rep(c("blue", "green", "red"), 4),
               ylab = "Bias", text = "(A7) ES crisis event")
## (3,2) SE
boxplot_groups(DimMat[73, 1:4],  DimMat[74, 1:4],  DimMat[75, 1:4],
               DimMat[76, 1:6],  DimMat[77, 1:6],  DimMat[78, 1:6],
               DimMat[79, 1:8],  DimMat[80, 1:8],  DimMat[81, 1:8],
               DimMat[82, 1:10], DimMat[83, 1:10], DimMat[84, 1:10],
               names = xlabels(c("MC", "HMC", "GS"),
                               info = c("d=4", "d=6", "d=8", "d=10")),
               col = rep(c("blue", "green", "red"), 4),
               ylab = "Standard error", text = "(A8) ES crisis event")
## (3,3) Time-adjusted MSE
boxplot_groups(DimMat[85, 1:4],  DimMat[86, 1:4],  DimMat[87, 1:4],
               DimMat[88, 1:6],  DimMat[89, 1:6],  DimMat[90, 1:6],
               DimMat[91, 1:8],  DimMat[92, 1:8],  DimMat[93, 1:8],
               DimMat[94, 1:10], DimMat[95, 1:10], DimMat[96, 1:10],
               names = xlabels(c("MC", "HMC", "GS"),
                               info = c("d=4", "d=6", "d=8", "d=10")),
               col = rep(c("blue", "green", "red"), 4),
               ylab = "Time-adjusted MSE", text = "(A9) ES crisis event")
par(opar) # restore plot parameters
layout(1) # restore layout
if(doPDF) dev.off.crop(file)


### Figure 5: Comparison in confidence level ###################################

if(doPDF) pdf(file = (file <- "ConfLelelPlot.pdf"), width = 10, height = 12)
## Layout
layout(matrix(1:9, ncol = 3, byrow = TRUE))
opar <- par(pty = "s")
## Plot
## (1,1) Bias
boxplot_groups(LevelMat[1,], LevelMat[2,],
               LevelMat[3,], LevelMat[4,],
               LevelMat[5,], LevelMat[6,],
               LevelMat[7,], LevelMat[8,],
               names = xlabels(c("MC", "HMC"),
                               info = c("0.9", "0.99", "0.999", "0.9999")),
               col = rep(c("blue", "green"), 4),
               ylab = "Bias", text = "(B1) VaR crisis event")
## (1,2) SE
boxplot_groups(LevelMat[9, ], LevelMat[10,],
               LevelMat[11,], LevelMat[12,],
               LevelMat[13,], LevelMat[14,],
               LevelMat[15,], LevelMat[16,],
               names = xlabels(c("MC", "HMC"),
                               info = c("0.9", "0.99", "0.999", "0.9999")),
               col = rep(c("blue", "green"), 4),
               ylab = "Standard error", text = "(B2) VaR crisis event")
## (1,3) Time-adjusted MSE
boxplot_groups(LevelMat[17,], LevelMat[18,],
               LevelMat[19,], LevelMat[20,],
               LevelMat[21,], LevelMat[22,],
               LevelMat[23,], LevelMat[24,],
               names = xlabels(c("MC", "HMC"),
                               info = c("0.9", "0.99", "0.999", "0.9999")),
               col = rep(c("blue", "green"), 4),
               ylab = "Time-adjusted MSE", text = "(B3) VaR crisis event")
## (2,1) Bias
boxplot_groups(LevelMat[25,], LevelMat[26,], LevelMat[27,],
               LevelMat[28,], LevelMat[29,], LevelMat[30,],
               LevelMat[31,], LevelMat[32,], LevelMat[33,],
               LevelMat[34,], LevelMat[35,], LevelMat[36,],
               names = xlabels(c("MC", "HMC", "GS"),
                               info = c("0.9, 0.9999", "(0.9, 0.99)", "(0.99, 0.999)", "(0.999, 0.9999)")),
               col = rep(c("blue", "green", "red"), 4),
               ylab = "Bias", text = "(B4) RVaR crisis event")
## (2,2) SE
boxplot_groups(LevelMat[37,], LevelMat[38,], LevelMat[39,],
               LevelMat[40,], LevelMat[41,], LevelMat[42,],
               LevelMat[43,], LevelMat[44,], LevelMat[45,],
               LevelMat[46,], LevelMat[47,], LevelMat[48,],
               names = xlabels(c("MC", "HMC", "GS"),
                               info = c("0.9, 0.9999", "(0.9, 0.99)", "(0.99, 0.999)", "(0.999, 0.9999)")),
               col = rep(c("blue", "green", "red"), 4),
               ylab = "Standard error", text = "(B5) RVaR crisis event")
## (2,3) Time-adjusted MSE
boxplot_groups(LevelMat[49,], LevelMat[50,], LevelMat[51,],
               LevelMat[52,], LevelMat[53,], LevelMat[54,],
               LevelMat[55,], LevelMat[56,], LevelMat[57,],
               LevelMat[58,], LevelMat[59,], LevelMat[60,],
               names = xlabels(c("MC", "HMC", "GS"),
                               info = c("0.9, 0.9999", "(0.9, 0.99)", "(0.99, 0.999)", "(0.999, 0.9999)")),
               col = rep(c("blue", "green", "red"), 4),
               ylab = "Time-adjusted MSE", text = "(B6) RVaR crisis event")
## (3,1) Bias
boxplot_groups(LevelMat[61,], LevelMat[62,], LevelMat[63,],
               LevelMat[64,], LevelMat[65,], LevelMat[66,],
               LevelMat[67,], LevelMat[68,], LevelMat[69,],
               LevelMat[70,], LevelMat[71,], LevelMat[72,],
               names = xlabels(c("MC", "HMC", "GS"),
                               info = c("0.9", "0.99", "0.999", "0.9999")),
               col = rep(c("blue", "green", "red"), 4),
               ylab = "Bias", text = "(B7) ES crisis event")
## (3,2) SE
boxplot_groups(LevelMat[73,], LevelMat[74,], LevelMat[75,],
               LevelMat[76,], LevelMat[77,], LevelMat[78,],
               LevelMat[79,], LevelMat[80,], LevelMat[81,],
               LevelMat[82,], LevelMat[83,], LevelMat[84,],
               names = xlabels(c("MC", "HMC", "GS"),
                               info = c("0.9", "0.99", "0.999", "0.9999")),
               col = rep(c("blue", "green", "red"), 4),
               ylab = "Standard error", text = "(B8) ES crisis event")
## (3,3) Time-adjusted MSE
boxplot_groups(LevelMat[85,], LevelMat[86,], LevelMat[87,],
               LevelMat[88,], LevelMat[89,], LevelMat[90,],
               LevelMat[91,], LevelMat[92,], LevelMat[93,],
               LevelMat[94,], LevelMat[95,], LevelMat[96,],
               names = xlabels(c("MC", "HMC", "GS"),
                               info = c("0.9", "0.99", "0.999", "0.9999")),
               col = rep(c("blue", "green", "red"), 4),
               ylab = "Time-adjusted MSE", text = "(B9) ES crisis event")
par(opar) # restore plot parameters
layout(1) # restore layout
if(doPDF) dev.off.crop(file)


### Figure 6: Comparison in HMC parameters #####################################

if(doPDF) pdf(file = (file <- "HMCparamPlot.pdf"), width = 10, height = 12)
## Layout
layout(matrix(1:9, ncol = 3, byrow = TRUE))
opar <- par(pty = "s")
## Plot
## (1,1) Bias
boxplot_basic(OptMat[1,], OptMat[2,], OptMat[3,], OptMat[4,], OptMat[5,], OptMat[6,],
              names = c("MC", "HMC, opt", "HMC, mm", "HMC, md", "HMC, dm","HMC, dd"),
              col = c("blue", rep("green", 5)), ylab = "Bias",
              text = "(C1) VaR crisis event")
## (1,2) SE
boxplot_basic(OptMat[7,], OptMat[8,], OptMat[9,], OptMat[10,], OptMat[11,], OptMat[12,],
              names = c("MC", "HMC, opt", "HMC, mm", "HMC, md", "HMC, dm","HMC, dd"),
              col = c("blue", rep("green", 5)), ylab = "Standard error",
              text = "(C2) VaR crisis event")
## (1,3) Time-adjusted MSE
boxplot_basic(OptMat[13,], OptMat[14,], OptMat[15,], OptMat[16,], OptMat[17,], OptMat[18,],
              names = c("MC", "HMC, opt", "HMC, mm", "HMC, md", "HMC, dm","HMC, dd"),
              col = c("blue", rep("green", 5)), ylab = "Time-adjusted MSE",
              text = "(C3) VaR crisis event")
## (2,1) Bias
boxplot_basic(OptMat[19,], OptMat[20,], OptMat[21,], OptMat[22,], OptMat[23,], OptMat[24,],
              names = c("MC", "HMC, opt", "HMC, mm", "HMC, md", "HMC, dm","HMC, dd"),
              col = c("blue", rep("green", 5)), ylab = "Bias",
              text = "(C4) RVaR crisis event")
## (2,2) SE
boxplot_basic(OptMat[25,], OptMat[26,], OptMat[27,], OptMat[28,], OptMat[29,], OptMat[30,],
              names = c("MC", "HMC, opt", "HMC, mm", "HMC, md", "HMC, dm","HMC, dd"),
              col = c("blue", rep("green", 5)), ylab = "Standard error",
              text = "(C5) RVaR crisis event")
## (2,3) Time-adjusted MSE
boxplot_basic(OptMat[31,], OptMat[32,], OptMat[33,], OptMat[34,], OptMat[35,], OptMat[36,],
              names = c("MC", "HMC, opt", "HMC, mm", "HMC, md", "HMC, dm","HMC, dd"),
              col = c("blue", rep("green", 5)), ylab = "Time-adjusted MSE",
              text = "(C6) RVaR crisis event")
## (3,1) Bias
boxplot_basic(OptMat[37,], OptMat[38,], OptMat[39,], OptMat[40,], OptMat[41,], OptMat[42,],
              names = c("MC", "HMC, opt", "HMC, mm", "HMC, md", "HMC, dm","HMC, dd"),
              col = c("blue", rep("green", 5)), ylab = "Bias",
              text = "(C7) ES crisis event")
## (3,2) SE
boxplot_basic(OptMat[43,], OptMat[44,], OptMat[45,], OptMat[46,], OptMat[47,], OptMat[48,],
              names = c("MC", "HMC, opt", "HMC, mm", "HMC, md", "HMC, dm","HMC, dd"),
              col = c("blue", rep("green", 5)), ylab = "Standard error",
              text = "(C8) ES crisis event")
## (3,3) Time-adjusted MSE
boxplot_basic(OptMat[49,], OptMat[50,], OptMat[51,], OptMat[52,], OptMat[53,], OptMat[54,],
              names = c("MC", "HMC, opt", "HMC, mm", "HMC, md", "HMC, dm","HMC, dd"),
              col = c("blue", rep("green", 5)), ylab = "Time-adjusted MSE",
              text = "(C9) ES crisis event")
par(opar) # restore plot parameters
layout(1) # restore layout
if(doPDF) dev.off.crop(file)


